from sqlalchemy import Column, Integer, String
from oddasr.model.db import Base
from enum import Enum

class CMeetingStatus(str, Enum):
    MEETING_STATUS_APPLY = -1
    MEETING_STATUS_MINUTES = 0
    MEETING_STATUS_ENCODE = 1
    MEETING_STATUS_FINISH = 2
    MEETING_STATUS_ERROR = 3

    def __int__(self):
        if self.value == -1: return "MEETING_STATUS_APPLY"
        elif self.value == 0: return "MEETING_STATUS_MINUTES"
        elif self.value == 1: return "MEETING_STATUS_ENCODE"
        elif self.value == 2: return "MEETING_STATUS_FINISH"
        elif self.value == 3: return "MEETING_STATUS_ERROR"
        else: return "MEETING_STATUS_INIT"

class CMeetingStatus2(str, Enum):
    MEETING_OPEN_STATUS_INIT = 0
    MEETING_OPEN_STATUS_ENCODE = 1
    MEETING_OPEN_STATUS_FINISH = 2
    MEETING_OPEN_STATUS_EXCEPTION = 3

    def __int__(self):
        if self.value == -1: return "MEETING_OPEN_STATUS_INIT"
        elif self.value == 1: return "MEETING_OPEN_STATUS_ENCODE"
        elif self.value == 2: return "MEETING_OPEN_STATUS_FINISH"
        elif self.value == 3: return "MEETING_OPEN_STATUS_EXCEPTION"
        else: return "MEETING_OPEN_STATUS_INIT"

class CMeeting(Base):
    __tablename__ = 'oddasr_meeting'

    id = Column(Integer, primary_key=True)     
    session_id = Column(String(1024) ,unique=True)        # open->session_id uuid
    # meeting_e164 = Column(String(32))                       # priority
    meeting_type = Column(Integer)                          # 0 open-live 1 open-file 2 meeting
    meeting_alias = Column(String(1024))                    # filename
    meeting_addr = Column(String(1024))                     # alitaskId
    meeting_begin_time = Column(String(32))
    meeting_end_time = Column(String(32))
    meeting_participant = Column(String(1024))              # split by ',' , length > 1024?  uniqueid
    meeting_audio_res_id = Column(String(1024),default='')  
    meeting_status = Column(Integer)                        # -1 apply 0 ing 1 encode 2 finish 3 error
    meeting_op_status = Column(Integer , default=0)         # 0 , 1 for meeting.

    def __repr__(self):
        return ("<CMeeting(id:%d)>") % (self.id)


    


import json
import os
import platform
import os.path

from oddasr.log import logger

from oddasr.logic import sensitivewords
import oddasr.odd_asr_config as config


def get_sensitive_words_from_file():
    path = ''
    res = {}
    if platform.system() != "Windows":
        path = os.path.join(config.odd_asr_slp_cfg['sensitive_words']['path'], 'sensitivewords.txt')
    else:
        path = os.getcwd()
        path = os.path.join(path, '.\\download', '.\\sensitivewords.txt')
    logger.info(path)
    words = []
    if os.path.isfile(path):
        file = open(path, 'r', encoding='utf-8')
        lines = file.readlines()

        for line in lines:
            logger.info(line)
            line = line.strip()
            if len(line) != 0:
                words.append(line)
        file.close()
    if len(words) == 0 and platform.system() == "Windows":
        words.append('默认值1')
        words.append('默认值2')
    return words

def get_sensitive_words(sens_id):
    words = sensitivewords.SensitiveWordManage.get_sensitive_word(sens_id)
    rows = []
    if words:
        i = 1
        for word in json.loads(words['sensitive_word']):
            rows.append({'id': i, 'word': word})
            i = i + 1
    total = len(rows)
    res = {'total': total, 'rows': rows}
    return res


def save_sensitive_words(words):
    path = ''
    if platform.system() != "Windows":
        path = os.path.join(config.odd_asr_slp_cfg['sensitive_words']['path'], 'sensitivewords.txt')
    else:
        print(words)
        path = os.getcwd()
        path = os.path.join(path, '.\\download', '.\\sensitivewords.txt')
    file = open(path, 'w', encoding='utf-8')
    for word in words:
        word = word.strip()
        if len(word) != 0:
            file.write(word + '\n')
    file.close()
    return True

def update_sensitive_words(words):
    res = save_sensitive_words(words)
    return res

def merge_sensitive_words(unique_id, sensitive_word):
    logger.info("+++++++++++++++++++++")
    data = sensitivewords.SensitiveWordManage.set_sensitive_word(unique_id, sensitive_word)
    logger.info(f"set words{data}")
    '''
    敏感词通知到jdlserver
    '''
    # sensitive_words_ntf = {'msg_type': 'SENSITIVEWORDS_NTF', 'msg_id': '', 'service_type': 'ASR_MANAGER',
    #                        'msg_data': {'unique_id': unique_id}}

    # sync_send_to_jdl_server_ws(json_data=json.dumps(sensitive_words_ntf), reply=False)

    return True


def sync_sensitive_words(words):
    #jdlserver read from file, no need sync.
    return

if __name__ == '__main__':
    print(get_sensitive_words())


from email.policy import default
from sqlalchemy import Column, Integer, String, Text

from oddasr.model.db import Base

class CText(Base):
    __tablename__ = 'oddasr_text'
    
    text_id = Column(Integer, primary_key=True)     #
    session_id = Column(Text , index=True)
    text_bg_time = Column(Integer)
    text_ed_time = Column(Integer)
    text_content = Column(Text)        
    text_time = Column(Integer)
    text_term_e164 = Column(String(32))
    text_term_alias = Column(String(64))
    text_flag  = Column(Integer)            #1 句后分段 2 句前分段
    text_term_tb_time = Column(Integer)     # 有效片段未对齐前的tb
    text_term_td_time = Column(Integer)     # 有效片段未对齐前的td

    text_sign = Column(Integer,default=0)   #0 不需要优化的混音文本 
                                            #1 需要被优化的混音文本，尚未被优化掉
                                            #2 需要被优化的混音文本，已经被优化掉
                                            #3 优化成功的有效片段文本
                                            #---------------------------------
                                            # 0~3 TCMDAsrMinutesTextOptNtf (ASR_MINUTES_TEXT_OPT_NTF)消息通知
                                            # 4~6 TCMDASRMinutesTextOptVadNtf (ASR_MINUTES_TEXT_OPT_VAD_NTF) 消息通知
                                            #---------------------------------
                                            #4 未对齐的有效片段文本 带有tb/td
                                            #5 优化成功的有效片段文本 和 #3一样，但带有tb/td
                                            #6 未被用作优化的有效 带有td/td
    def __repr__(self):
        return ("<CText(id='%s' bg='%s' ed='%s' text='%s')") % (
          self.text_id ,self.text_bg_time , self.text_ed_time , self.text_content)



import oddasr.odd_asr_config as config

import io
import json

g_data = {'user':'Admin','pwd':'Odd_Asr'}

def _load(path):
    try:
        f = io.open(path , "r")
        all = f.read()
        j = json.loads(all)
        g_data['user'] = j['user']
        g_data['pwd'] = j['pwd']
    except:
        pass

_load(config.Users)

def _save(path ):
    f = io.open(path , 'w')
    all = json.dumps(g_data)
    f.write(all)

def check_user(user,pwd):
    if g_data['user'] == user and g_data['pwd'] == pwd:
        return True
    else:
        return False
    
def modify_user(user,oldpwd,newpwd):
    if g_data['user'] != user or g_data['pwd'] != oldpwd:
        return False
    else:
        g_data['user'] = user
        g_data['pwd'] = newpwd
        _save(config.Users)
        return True



from funasr import AutoModel


model = AutoModel(model="paraformer-zh-streaming", model_revision="v2.0.4")

import soundfile
import os

chunk_size = [0, 10, 5] #[0, 10, 5] 600ms, [0, 8, 4] 480ms
encoder_chunk_look_back = 4 #number of chunks to lookback for encoder self-attention
decoder_chunk_look_back = 1 #number of encoder chunks to lookback for decoder cross-attention

def stream_asr():
    wav_file = "1.wav"
    speech, sample_rate = soundfile.read(wav_file)
    chunk_stride = chunk_size[1] * 960 # 600ms

    cache = {}
    total_chunk_num = int(len((speech)-1)/chunk_stride+1)

    print(f"total_chunk_num: {total_chunk_num}, chunk_stride: {chunk_stride}")

    for i in range(total_chunk_num):
        speech_chunk = speech[i*chunk_stride:(i+1)*chunk_stride]
        is_final = i == total_chunk_num - 1
        res = model.generate(input=speech_chunk, cache=cache, is_final=is_final, chunk_size=chunk_size, encoder_chunk_look_back=encoder_chunk_look_back, decoder_chunk_look_back=decoder_chunk_look_back)
        print(res)


def stream_asr2():
    file = "1.wav"
    import numpy as np

    total_text = []
    cache = {}

    # 假设 PCM 文件是 16 位有符号整数，采样率为 16000 Hz
    dtype = np.int16

    try:
        # 以二进制模式读取 PCM 文件
        with open(file, 'rb') as f:
            pcm_data = np.frombuffer(f.read(), dtype=dtype)
    except Exception as e:
        print(f"Error reading PCM file: {e}")
        exit(1)

    # 将 PCM 数据转换为浮点数，范围在 -1.0 到 1.0 之间
    speech = pcm_data.astype(np.float32) / 32768.0

    chunk_stride = chunk_size[1] * 960 # 600ms
    total_chunk_num = len(speech) // chunk_stride

    print(f"total_chunk_num: {total_chunk_num}, chunk_stride: {chunk_stride}")

    for i in range(total_chunk_num):
        start = i * chunk_stride
        end = start + chunk_stride
        audio_chunk = speech[start:end]

        # 标记是否为最后一个分片
        is_final = i == total_chunk_num - 1
        text = model.generate(input=audio_chunk, cache=cache, is_final=is_final, chunk_size=chunk_size, encoder_chunk_look_back=encoder_chunk_look_back, decoder_chunk_look_back=decoder_chunk_look_back)
        if text and isinstance(text, list) and len(text) > 0 and 'text' in text[0]:
            partial_text = text[0]['text']
            print(f"Partial result for chunk {i + 1}/{total_chunk_num}: {partial_text}")
            total_text.append(partial_text)
        else:
            print(f"Empty or invalid result for chunk {i + 1}/{total_chunk_num}")

    # 处理剩余不足 960 个样本的数据
    if len(speech) % chunk_stride != 0:
        remaining_audio = speech[total_chunk_num * chunk_stride:]
        text = model.generate(input=remaining_audio, cache=cache, is_final=True, chunk_size=chunk_size, encoder_chunk_look_back=encoder_chunk_look_back, decoder_chunk_look_back=decoder_chunk_look_back)
        if text and isinstance(text, list) and len(text) > 0 and 'text' in text[0]:
            partial_text = text[0]['text']
            print(f"Partial result for remaining chunk: {partial_text}")
            total_text.append(partial_text)
        else:
            print(f"Empty or invalid result for remaining chunk")

    final_text = ''.join(total_text)
    print(f"Final transcription result: {final_text}")

if __name__ == "__main__":

    import argparse

    parser = argparse.ArgumentParser(description="Your WAV file need to recoginze to text.")
    parser.add_argument("test_type", type=str, help="test type, 0 for stream_asr, 1 for stream_asr2.")
    args = parser.parse_args()

    print(f"test type: {args.test_type}")

    if args.test_type == "0":
        stream_asr()
    else:
        stream_asr2()



import sys
import os

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

import oddasr.odd_asr_config as config

sys.path.append('../')

# 获取包安装目录
package_dir = os.path.dirname(os.path.abspath(__file__))
# 获取数据库文件绝对路径
db_path = os.path.join(package_dir, '..', config.db_cfg["db_name"])

if config.db_cfg["db_engine"] == "sqlite":
    # 使用绝对路径连接数据库
    e = create_engine(f"sqlite:///{db_path}", pool_recycle=3600, pool_pre_ping=True)
elif config.db_cfg["db_engine"] == "mysql":
    e = create_engine(
        "mysql+pymysql://" + config.db_cfg["db_user"] + ":" + config.db_cfg["db_password"] + "@" + config.db_cfg["db_host"] + "/" + config.db_cfg["db_name"] + "?charset=utf8mb4",
        pool_recycle=3600, pool_pre_ping=True)  # , echo=True)
elif config.db_cfg["db_engine"] == "openGauss":
    e = create_engine(
        'opengauss+psycopg2://' + config.db_cfg["db_user"] + ":" + config.db_cfg["db_password"] + "@"
        + config.db_cfg["db_host"] + ':' + config.db_cfg["db_port"] + "/" + config.db_cfg["db_name"],
        pool_recycle=3600, pool_pre_ping=True)
else:
    print("db_engine not support")
    exit(1)

Session = sessionmaker(bind=e)
Base = declarative_base()

def to_dict(self):
    return {c.name: getattr(self, c.name) for c in self.__table__.columns}


def to_json(all_vendors):
    v = [ven.to_dict() for ven in all_vendors]
    return v


"""热词相关处理逻辑"""
from sqlalchemy import and_
import json

from oddasr.log import logger

from oddasr.model import db
from oddasr.model import hotword
from oddasr.logic.users import check_user,g_data
import oddasr.logic.odd_asr_exceptions as exceptions

class HotWordManage(object):

    @classmethod
    def set_hot_word(cls, hotwords_id, hotwords_type, words):
        db_session = db.Session()

        hot_word_m = db_session.query(hotword.CHotWords).filter(
            and_(hotword.CHotWords.hotwords_id == hotwords_id, hotword.CHotWords.hotwords_type == hotwords_type)
        ).first()
        if hot_word_m:
            hot_word_m.words = json.dumps(words, ensure_ascii=False)
        else:
            hot_word_m = hotword.CHotWords(
                hotwords_id=hotwords_id,
                hotwords_type=hotwords_type,
                words=json.dumps(words, ensure_ascii=False),
            )
        db_session.merge(hot_word_m)
        db_session.commit()
        data = db.to_dict(hot_word_m)
        db_session.close()
        return data

    @classmethod
    def get_all_hot_word(cls):
        db_session = db.Session()
        hot_word_m = db_session.query(hotword.CHotWords).all()
        data = []
        if hot_word_m:
            for hot_word_item in hot_word_m:
                words = hot_word_item.words
                logger.info(f"wordswords:{words}")
                for word in json.loads(words):
                    if hot_word_item.hotwords_type == 2 and len(word) > 0:
                        words = list(word)[0]
                    else:
                        words = word
                    data.append({
                        "type": hot_word_item.hotwords_type,
                        "words": words
                    })
            db_session.close()
        else:
            db_session.close()
            logger.error("hotwords_id not exist")
            raise exceptions.ResultException(error_desc="hotwords_id not exist")
        return data

    @classmethod
    def get_hot_word(cls, hotwords_id, hotwords_type):
        db_session = db.Session()
        hot_word_m = db_session.query(hotword.CHotWords).filter(
            and_(hotword.CHotWords.hotwords_id == hotwords_id, hotword.CHotWords.hotwords_type == hotwords_type)
        ).first()
        if hot_word_m:
            data = db.to_dict(hot_word_m)
            db_session.close()
        else:
            db_session.close()
            logger.error("hotwords_id not exist")
            raise exceptions.ResultException(error_desc="hotwords_id not exist")
        return data

    @classmethod
    def del_hot_word(cls, unique_id):
        db_session = db.Session()
        hot_word_m = db_session.query(hotword.CHotWords).filter(hotword.CHotWords.unique_id == unique_id).first()
        if not hot_word_m:
            raise exceptions.ResultException(error_desc="hotwords_id not exist")
        db_session.delete(hot_word_m)
        db_session.commit()
        db_session.close()
        return {}

    @classmethod
    def get_hot_word_for_opensdk(cls, hotwords_id, hotwords_type):
        db_session = db.Session()
        hot_word_m = db_session.query(hotword.CHotWords).filter(
            and_(hotword.CHotWords.hotwords_id == hotwords_id, hotword.CHotWords.hotwords_type == hotwords_type)
        ).first()
        if hot_word_m:
            data = db.to_dict(hot_word_m)
            db_session.close()
        else:
            db_session.close()
            data = {}
        return data

    ####################################################
    ### for hotwords new interfaces
    ####################################################
    @classmethod
    def get_hot_word_list(cls, uniqueid):
        db_session = db.Session()
        hot_word_m = db_session.query(hotword.CHotWords).filter(hotword.CHotWords.hotwords_id == uniqueid).first()
        if hot_word_m:
            data = db.to_dict(hot_word_m)
            db_session.close()
        else:
            db_session.close()
            logger.error("uniqueid not exist")
            raise exceptions.ResultException(error_desc="uniqueid not exist")
        return data

    @classmethod
    def get_hotwords_group(cls, userid):
        error_code = 0
        error_desc = ''
        if (g_data['user'] != userid):
            logger.error('invalid user access!')
            raise exceptions.ResultException(error_desc='invalid user access')

        data = g_data['data']

        return data

    @classmethod
    def submit_hotwords_group(cls, userid, datas):
        error_code = 0
        error_desc = ''

        print(datas)

        if (g_data['user'] != userid):
            logger.error('invalid user access!')
            raise exceptions.ResultException(error_desc='invalid user access')

        # format string to g_data format
        jj = []
        for item in datas:
            j = json.loads(item)
            # check new data type
            dict_obj = {"id": j["id"], "hotword_type": j["hotword_type"], "name": j["name"]}
            jj.append(dict_obj)

        g_data['data'] = jj

        # save to data.json
        str = json.dumps(g_data)
        f = open("data.json", "w+")
        f.write(str)
        f.close()

        return jj;

    @classmethod
    def save_hotwords(cls, uniqueid, hotwords_type, words):
        db_session = db.Session()

        hot_word_m = db_session.query(hotword.CHotWords).filter(hotword.CHotWords.hotwords_id == uniqueid).first()
        if hot_word_m:
            if words:
                hot_word_m.words = json.dumps(words, ensure_ascii=False)
                print("hot_word_m.words=%s" % hot_word_m.words)
                status = 0
        else:
            hot_word_m = hotword.CHotWords(
                hotwords_id=uniqueid,
                hotwords_type=hotwords_type,
                words=json.dumps(words, ensure_ascii=False),
                status=0,
            )

        print(hot_word_m)

        db_session.merge(hot_word_m)
        db_session.commit()
        data = db.to_dict(hot_word_m)
        db_session.close()
        return data



import threading
import time
import json
import uuid
from sqlalchemy import false

from oddasr.log import logger

from oddasr.logic.minutes import *
from oddasr.logic import sensitivewords
from oddasr.logic.odd_asr_file import OddAsrFile
from oddasr.logic.odd_asr_instance import find_free_odd_asr_file, init_instance_file

from oddasr.model.meeting import CMeetingStatus2 # MEETING_OPEN_STATUS_INIT, MEETING_OPEN_STATUS_ENCODE, MEETING_OPEN_STATUS_EXCEPTION

class ScheduledTask(threading.Thread):
    def __init__(self, status_notifier):
        self.is_transmit = False
        self.thread_run = True
        self.encode_task = False
        self.wait_time = 10
        self.status_notifier = status_notifier # 转写成功/失败通知回调
        threading.Thread.__init__(self)
        threading.Thread.setDaemon(self, True)

    @staticmethod
    def start_task(priority, unique_id, file_name):
        str_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        session_id = uuid.uuid1()
        res = CStorageMinutes.addOpenFileMeeting(strSessionId=session_id, strRank=unique_id, strUniqueId=unique_id, strFileName=file_name, strBeginTime=str_time)
        if res:
            return session_id
        else:
            return None

    @staticmethod
    def cancel_task(session_id):
        logger.debug(f"cancel_task {session_id}")
        task_info = CStorageMinutes.getOpenFileMeetingInfo(session_id)
        if task_info is None:
            return True
        elif task_info["status"] == CMeetingStatus2.MEETING_OPEN_STATUS_INIT:
            return CStorageMinutes.deleteOpenFileMeeting(session_id)
        else:
            logger.info("task can't cancel status " + str(task_info["status"]))
        return False

    @staticmethod
    def task_status(session_id):
        logger.debug(f"task_status {session_id}")
        task_info = CStorageMinutes.getOpenFileMeetingInfo(session_id)
        if task_info is None or task_info["status"] == CMeetingStatus2.MEETING_OPEN_STATUS_EXCEPTION:
            return {"texts": {}, "status": 3}
        if task_info["status"] == CMeetingStatus2.MEETING_OPEN_STATUS_INIT:
            return {"texts": {}, "status": 0}
        elif task_info["status"] == CMeetingStatus2.MEETING_OPEN_STATUS_ENCODE:
            return {"texts": {}, "status": 1}
        else:
            record_texts, count = CStorageMinutes.getMeetingText(session_id, 0, 0 , False)
            texts = []
            for item in record_texts:
                text = {"bg": item.text_bg_time, "ed": item.text_ed_time, "text": item.text_content}
                texts.append(text)
            return {"texts": texts, "status": 2}

    @staticmethod
    def task_finish(session_id, texts):
        logger.debug(f"task_finish {session_id} {texts}")
        if len(texts) > 0: 
            task_info = CStorageMinutes.getOpenFileMeetingInfo(session_id)
            if task_info is not None:
                #uniqueId = hot_uid + ";" + sensitive_uid
                sensitive_uid = ""
                vec = task_info["uniqueId"].split(';')
                if len(vec) > 1:
                    sensitive_uid = vec[1]
                data = sensitivewords.SensitiveWordManage.get_sensitive_word(sensitive_uid)
                if data != None:
                    words = eval(data.get('sensitive_word', None))
                    if words is not None:
                        for i in range(len(texts)):
                            for j in range(len(words)):
                                texts[i]["text_content"] = texts[i]["text_content"].replace(words[j], "(" + words[j] + ")")
        str_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        return CStorageMinutes.updateOpenFileMeetingEnd(session_id, str_time, texts)    

    def run(self):
        logger.info(f"asr_recorder threadid: " + str(threading.get_ident()))
        while self.thread_run:
            # 重启时检查一下，是否有未完成的转写任务

            start_task_info = CStorageMinutes.getOpenFileMeetingInfoByPriority()

            if start_task_info is None:  # 无转写任务
                time.sleep(self.wait_time)
            else:
                session_id = start_task_info["uuid"]
                logger.debug(f"run {start_task_info}, session_id={session_id}")
                asr_file = self.__find_asr_instance(session_id=session_id)
                if asr_file is None:  # 资源申请失败
                    logger.info("appy resource fail " + str(start_task_info))
                    time.sleep(self.wait_time)
                else:
                    #uniqueId = hotowrds_list + ";" + sensiword_list
                    sensiword_list = ""
                    hotowrds_list = ""
                    vec = start_task_info["uniqueId"].split(';')
                    if len(vec) > 1:
                        sensiword_list = vec[1]
                    if len(vec) > 0:
                        hotowrds_list = vec[0]

                    output_text = asr_file.transcribe_file(start_task_info["fileName"], hotwords=hotowrds_list, output_format="raw")
                    self.__release_asr_instance(session_id, asr_file)  # 释放申请的ASR资源

                    if output_text is None:
                        logger.info("transcribe file fail " + str(start_task_info))
                        time.sleep(self.wait_time)
                    else:  # 设置任务为开始转写状态
                        logger.info(f"transcribe file success: {output_text}")

                        texts = self.__convert_texts(session_id=session_id, output_text=output_text)

                        # replace sensitive words
                        data = sensitivewords.SensitiveWordManage.get_sensitive_word(sensiword_list)
                        if data != None:
                            words = eval(data.get('sensitive_word', None))
                            if words is not None:
                                for i in range(len(texts)):
                                    for j in range(len(words)):
                                        texts[i]["text_content"] = texts[i]["text_content"].replace(words[j], "(" + words[j] + ")")
                        str_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
                        ret = CStorageMinutes.updateOpenFileMeetingEnd(strSessionId=session_id, strEndTime=str_time, arrTexts=texts)
                        if ret:
                            logger.info(f"update open file meeting end success: {ret}")
                        else:
                            logger.error(f"update open file meeting end fail: {ret}")

                    continue


    def __convert_texts(self, session_id, output_text):
        logger.debug(f"__convert_texts {output_text}, session_id={session_id}, type={type(output_text)}")

        if type(output_text) is str:
            try:
                output_text = json.loads(output_text)
            except json.JSONDecodeError:
                logger.error(f"Failed to parse output_text as JSON: {output_text}")
                return []
        
        if ("sentence_info" not in output_text[0]) or (len(output_text[0]["sentence_info"]) == 0):
            return [{"session_id": session_id, "text_bg_time": 0, "text_ed_time": 0, "text_content": output_text[0]["text"]}]
        
        texts = []
        output_text = output_text[0]["sentence_info"]
        for item in output_text:
            text = {
                        "session_id": session_id, 
                        "text_bg_time": item["start"], 
                        "text_ed_time": item["end"], 
                        "text_content": item["text"],
                        # "text_time": item["timestamp"],
                        "text_term_alias": f"spk {item.get('spk', 'unknown')}"
                    }  
            texts.append(text)
        return texts

    def __find_asr_instance(self, session_id):
        logger.info("find asr instance " + session_id)
        odd_asr_file: OddAsrFile = find_free_odd_asr_file()
        if odd_asr_file is None:
            logger.info("no free asr instance for session: " + str(session_id))
            return None
        else:
            odd_asr_file.set_busy(session_id)
            return odd_asr_file


    def __release_asr_instance(self, session_id, odd_asr_file: OddAsrFile):
        logger.info(f"release asr instance: {session_id}, instance: {odd_asr_file}")
        if not odd_asr_file.is_busy():  # 资源无效，勿须释放
            return True
        odd_asr_file.set_busy(False)
        return True

if __name__ == '__main__':
    from odd_asr_file import OddAsrFile, OddAsrParamsFile
    import odd_asr_config as config

    odd_asr_params_file = OddAsrParamsFile()
    odd_asr_file_set = set()

    def init_instance_file():
        global odd_asr_file_set
        for i in range(config.odd_asr_cfg["asr_file_cfg"]["max_instance"]):
            odd_asr_file = OddAsrFile(odd_asr_params_file)
            odd_asr_file_set.add(odd_asr_file)

    def find_free_odd_asr_file():
        '''
        find a free odd_asr_file
        :param :
        :return:
        '''
        global odd_asr_file_set
        for odd_asr_file in odd_asr_file_set:
            if not odd_asr_file.is_busy():
                return odd_asr_file
            
        return None

    init_instance_file()

    scheduled_task = ScheduledTask(status_notifier=None)

    # priority = 1
    # unique_id = "220ed89d-7b84-11f0-9b0f-5c879c002fd9"
    # file_name = "test_cn_male_9s.wav"
    # scheduled_task.start_task(priority, unique_id, file_name)

    scheduled_task.start()

    try:
        # 主循环保持程序运行
        while True:
            time.sleep(60)  # 每分钟检查一次
    except KeyboardInterrupt:
        # 处理Ctrl+C中断，优雅退出
        logger.info("接收到中断信号，准备退出程序...")
        scheduled_task.thread_run = False  # 设置标志让线程退出
        scheduled_task.join(timeout=5)  # 等待线程结束，最多等待5秒
        logger.info("程序已退出")




# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: odd_asr_exceptions.py 
@info: 消息模版
"""

from flask import jsonify

from oddasr.logic import odd_asr_result

# 以下是c++的错误的PYTHON实现.

# Format: A.BBB.C.DDD

# A: TYPE [1,9]
EM_ERR_TYPE_C = "1"         #caller error
EM_ERR_TYPE_S = "2"         #callee error
EM_ERR_TYPE_T = "3"   

# BBB: MOD [000,999]
EM_ERR_MOD_LLM = "001"
EM_ERR_MOD_ASR = "002"
EM_ERR_MOD_TTS = "003"
EM_ERR_MOD_EMOTION = "004"
EM_ERR_MOD_MEMORY = "005"

# C.DDD: CODE [0000,9999]

g_mai_err_api = {}

def DEF_ERR(MOD,TYPE,CODE,DESC = ""):
    error_code = (int)(TYPE + MOD + CODE)
    g_mai_err_api[error_code] = DESC
    return error_code

def mai_err_name(error_code):
    ns = globals()
    for name in ns:
        if ns[name] == error_code:
            return name
    return ""

def mai_err_desc(error_code):
    if error_code in g_mai_err_api:
        return g_mai_err_api[error_code]
    return ""


# 以下是错误码定义:
## 通用错误：
## 错误码 原因 解决办法 
EM_ERR_ASR_AUTH_ERROR       = 40000001 # 身份认证失败 检查使用的令牌是否正确，是否过期
EM_ERR_ASR_INVALID_REQ      = 40000002 # 无效的消息 检查发送的消息是否符合要求
EM_ERR_ASR_INVALID_TOKEN    = 40000003 # 令牌过期或无效的参数 首先检查使用的令牌是否过期，然后检查参数值设置是否合理
EM_ERR_ASR_SESSION_TIMEDOUT = 40000004 # 空闲超时 确认是否长时间没有发送数据到服务端
EM_ERR_ASR_REQ_TOO_MANY     = 40000005 # 请求数量过多 检查是否超过了并发连接数或者每秒钟请求数
EM_ERR_ASR_CLIENT_ERROR     = 40000006 # 默认的客户端错误码 查看错误消息或提交工单
EM_ERR_ASR_SERVER_ERROR     = 50000000 # 默认的服务端错误 如果偶现可以忽略，重复出现请提交工单
EM_ERR_ASR_GRPC_ERROR       = 50000001 # 内部 GRPC 调用错误 如果偶现可以忽略，重复出现请提交工单
EM_ERR_ASR_GRPC_ERROR_2     = 52010001 # 内部 GRPC 调用错误 如果偶现可以忽略，重复出现请提交工单


## 网关错误：
## 错误码 原因 解决办法 
EM_ERR_ASR_GATEWAY_ERROR_1 = 40010001 # 不支持的接口 使用了不支持的接口，如果使用 SDK 请提交工单
EM_ERR_ASR_GATEWAY_ERROR_2 = 40010002 # 不支持的指令 使用了不支持的指令，如果使用 SDK 请提交工单
EM_ERR_ASR_GATEWAY_ERROR_3 = 40010003 # 无效的指令 指令格式错误，如果使用 SDK 请提交工单
EM_ERR_ASR_GATEWAY_ERROR_4 = 40010004 # 客户端提前断开连接 检查是否在请求正常完成之前关闭了连接
EM_ERR_ASR_GATEWAY_ERROR_5 = 40010005 # 任务状态错误 发送了当前任务状态不能处理的指令


## Meta 错误
## 错误码 原因 解决办法 
EM_ERR_ASR_META_ERROR_5 = 40020105 # 应用不存在 检查应用 appKey 是否正确，是否与令牌归属同一个账号


## 实时语音识别
## 错误码 原因 解决办法 
EM_ERR_ASR_REAL_TIME_CLIENT_ERROR_1 = 41040201 # 客户端 10s 内停止发送数据 检查网络问题，或者检查业务中是否存在不发数据的情况
EM_ERR_ASR_REAL_TIME_CLIENT_ERROR_2 = 41040202 # 客户端发送数据过快，服务器资源已经耗尽 检测客户端发包是否过快，是否按照 1：1 的实时率来发包
EM_ERR_ASR_REAL_TIME_CLIENT_ERROR_3 = 41040203 # 客户端发送音频格式不正确 请将音频数据的格式转换为 SDK 目前支持的音频格式来发包
EM_ERR_ASR_REAL_TIME_CLIENT_ERROR_4 = 41040204 # 客户端调用方法异常 客户端应该先调用发送请求接口，在发送请求完毕后再调用其他接口
EM_ERR_ASR_REAL_TIME_CLIENT_ERROR_5 = 41040205 # 客户端设置 MAXSILENCE_PARAM 方法异常 参数 MAXSILENCE_PARAM 的范围在[200-2000]
EM_ERR_ASR_REAL_TIME_SERVER_ERROR_1 = 51040101 # 服务端内部错误 未知错误
EM_ERR_ASR_REAL_TIME_SERVER_ERROR_2 = 51040102 # 保留参数 ASR_SERVICE_UNAVAILABLE
EM_ERR_ASR_REAL_TIME_SERVER_ERROR_3 = 51040103 # 实时语音识别服务不可用 需要查看实时语音识别服务是否有任务堆积等导致任务提交失败
EM_ERR_ASR_REAL_TIME_SERVER_ERROR_4 = 51040104 # 请求实时语音识别服务超时 具体需要排查实时语音识别日志
EM_ERR_ASR_REAL_TIME_SERVER_ERROR_5 = 51040105 # 调用实时语音识别服务失败 检查实时语音识别服务是否启动，端口是否正常开启
EM_ERR_ASR_REAL_TIME_SERVER_ERROR_6 = 51040106 # 实时语音识别服务负载均衡失败，未获取到实时语音识别服务的 IP 地址检查 VPC 中的实时语音识别服务机器是否有异常


#错误码定义如下:
## client errors
EM_ERR_ASR_ARGS_ERROR                             = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_C , "0001")
EM_ERR_ASR_SESSION_ID_EXIST                       = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_C , "0002")
EM_ERR_ASR_SESSION_ID_NOVALID                     = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_C , "0003")
EM_ERR_ASR_MINUTES_MOID_ERROR                     = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_C , "0004")
## server errors
EM_ERR_ASR_SERVER_ERROR                           = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_S , "0001", "timeout")
EM_ERR_ASR_WS_NONE                                = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_S , "0002", "reply timeout")
EM_ERR_ASR_SYNC_HOTWORDS_ERROR                    = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_S , "0003")
EM_ERR_ASR_METHOD_NOT_SUPPORT                     = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_S , "0004")
EM_ERR_ASR_MINUTES_STATUS_ERROR                   = DEF_ERR(EM_ERR_MOD_ASR, EM_ERR_TYPE_S , "0005")


class CodeException(Exception):

    def __init__(self, error_code, error_desc):
        super().__init__()
        self.error_code = error_code
        self.error_desc = error_desc

    def __str__(self):
        return "%d - %s" % (self.error_code, self.error_desc)

    def __unicode__(self):
        return u"%d - %s" % (self.error_code, self.error_desc)


class ResultException(CodeException):
    """异常返回"""
    def __init__(self, error_code=EM_ERR_ASR_ARGS_ERROR, error_desc=mai_err_name(EM_ERR_ASR_ARGS_ERROR)):
        super(ResultException, self).__init__(error_code, error_desc)


def handler(exc):
    return jsonify(odd_asr_result.from_exc(exc))



# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: odd_asr_instance.py 
@info: ASR实例管理
"""

from oddasr.log import logger

from oddasr.logic.odd_asr_file import OddAsrFile, OddAsrParamsFile
from oddasr.logic.odd_asr_sentence import OddAsrSentence, OddAsrParamsSentence
import oddasr.odd_asr_config as config

# File ASR
odd_asr_params_file = OddAsrParamsFile()
odd_asr_file_set = set()

def init_instance_file():
    global odd_asr_file_set
    for i in range(config.odd_asr_cfg["asr_file_cfg"]["max_instance"]):
        odd_asr_file = OddAsrFile(odd_asr_params_file)
        odd_asr_file_set.add(odd_asr_file)

def find_free_odd_asr_file():
    '''
    find a free odd_asr_file
    :param :
    :return:
    '''
    global odd_asr_file_set
    for odd_asr_file in odd_asr_file_set:
        if not odd_asr_file.is_busy():
            return odd_asr_file
        
    return None

# Sentence ASR
odd_asr_params_sentence = OddAsrParamsSentence()
odd_asr_sentence_set = set()

def init_instance_sentence():
    global odd_asr_sentence_set
    for i in range(config.odd_asr_cfg["asr_sentence_cfg"]["max_instance"]):
        odd_asr_sentence = OddAsrSentence(odd_asr_params_sentence)
        odd_asr_sentence_set.add(odd_asr_sentence)

def find_free_odd_asr_sentence():
    '''
    find a free odd_asr_sentence
    :param :
    :return:
    '''
    global odd_asr_sentence_set
    for odd_asr_sentence in odd_asr_sentence_set:
        if not odd_asr_sentence.is_busy():
            return odd_asr_sentence
        
    return None



# -*- coding: utf-8 -*-
""" 
@author: catherine@oddmeta.com 
@software: PyCharm 
@file: sensitiveword.py 
@time: 2021/12/1 16:25 
@info: 敏感词数据库模型
"""

from sqlalchemy import Column, Integer, String, Text
from sqlalchemy.dialects.mysql import MEDIUMTEXT, LONGTEXT

from oddasr.model import db


class CSensitiveWord(db.Base):
    __tablename__ = 'oddasr_sensitive_words'
    id = Column(Integer, primary_key=True)
    unique_id = Column(String(64), unique=True)
    sensitive_word = Column(Text)

    def __repr__(self):
        return f'<CSensitiveWord {self.unique_id!r}>'



# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: odd_asr.py 
@info: 消息模版
"""

import os
import uuid
import wave

from flask import Blueprint, request, jsonify
from mutagen.mp3 import MP3

# import app
from oddasr.log import logger
from oddasr.logic.odd_asr_instance import find_free_odd_asr_file, find_free_odd_asr_sentence
from oddasr.logic.scheduled_task import ScheduledTask
from oddasr.logic.odd_asr_file import OddAsrFile
from oddasr.logic.odd_asr_sentence import OddAsrSentence

########################################
## main
########################################
bp = Blueprint('asr', __name__, url_prefix='')

@bp.route('/v1/asr/sentence', methods=['POST'])
def transcribe_sentence():
    """
    Receive an audio file from the client and return the transcribed text.
    """
    return_ok = True
    try:

        # Get the uploaded audio file
        audio_file = request.files.get('audio')
        if not audio_file:
            return jsonify({"error": "Invalid parameter. audio param is required."}), 400
        
        # get mode from request if provided
        mode = request.form.get('mode', "file")  # mode should be a string like 'file', 'stream', 'pipeline'
        output_format = request.form.get('output_format', "txt")  # output_format should be a string like 'txt', 'srt', 'spk'
        hotwords = request.form.get('hotwords', "")  # hotwords should be a string like 'word1 word2'

        # Create a temporary directory with proper permissions
        import tempfile
        temp_dir = tempfile.gettempdir()
        temp_path = os.path.join(temp_dir, f"temp_audio_{os.urandom(8).hex()}.wav")

        # Save the audio file to a temporary location
        try:
            audio_file.save(temp_path)
            logger.info(f"Received audio and saved to: {temp_path}")
        except Exception as e:
            logger.error(f"Failed to save audio file: {e}")
            return jsonify({"error": f"Failed to save audio file: {str(e)}"}), 500

        # find a odd_asr_file instance
        odd_asr_sentence: OddAsrSentence = find_free_odd_asr_sentence()
        
        if not odd_asr_sentence:
            # Clean up the temporary file
            if os.path.exists(temp_path):
                os.remove(temp_path)
            return_ok = False
            result = "no available asr instance."
            return jsonify({"error": result}), 500

        # recognition with hotwords
        try:
            if mode == "file":
                result = odd_asr_sentence.transcribe_sentence(audio_file=temp_path, hotwords=hotwords, output_format=output_format)
            else:
                return_ok = False
                result = f"unsupported mode: {mode}."
        except Exception as e:
            logger.error(f"ASR processing error: {e}")
            result = f"ASR processing error: {str(e)}"
            return_ok = False
        finally:
            # Delete the temporary file
            if os.path.exists(temp_path):
                try:
                    os.remove(temp_path)
                    logger.info(f"Deleted temporary file: {temp_path}")
                except Exception as e:
                    logger.warning(f"Failed to delete temporary file: {e}")

        logger.info(f"Recognized mode:{mode}, fmt={output_format}, result: {result[:100]}...")

        # Return the recognition result
        if not return_ok:
            return jsonify({"error": result}), 500
        else:
            return jsonify({"text": result}), 200

    except Exception as e:
        logger.error(f"Unexpected error in transcribe endpoint: {e}")
        import traceback
        logger.error(traceback.format_exc())
        return jsonify({"error": str(e)}), 500

@bp.route('/v1/asr/file', methods=['POST'])
def transcribe():
    """
    Receive an audio file from the client and return the transcribed text.
    """
    return_ok = True
    try:

        # Get the uploaded audio file
        audio_file = request.files.get('audio')
        if not audio_file:
            return jsonify({"error": "Invalid parameter. audio param is required."}), 400
        
        # get mode from request if provided
        mode = request.form.get('mode', "file")  # mode should be a string like 'file', 'stream', 'pipeline'
        output_format = request.form.get('output_format', "txt")  # output_format should be a string like 'txt', 'srt', 'spk'
        hotwords = request.form.get('hotwords', "")  # hotwords should be a string like 'word1 word2'

        # Create a temporary directory with proper permissions
        import tempfile
        temp_dir = tempfile.gettempdir()
        temp_path = os.path.join(temp_dir, f"temp_audio_{os.urandom(8).hex()}.wav")

        # Save the audio file to a temporary location
        try:
            audio_file.save(temp_path)
            logger.info(f"Received audio and saved to: {temp_path}")
        except Exception as e:
            logger.error(f"Failed to save audio file: {e}")
            return jsonify({"error": f"Failed to save audio file: {str(e)}"}), 500


        logger.info(f"mode:{mode}, fmt={output_format}...")

        result = {

        }

        # Return the recognition result
        if not return_ok:
            return jsonify({"error": result}), 500
        else:
            return jsonify({"text": result}), 200

    except Exception as e:
        logger.error(f"Unexpected error in transcribe endpoint: {e}")
        import traceback
        logger.error(traceback.format_exc())
        return jsonify({"error": str(e)}), 500

@bp.route('/v1/asr/file/upload', methods=['POST'])
def file_upload():
    return jsonify({"text": "OK"}), 200

@bp.route('/v1/asr/task/delete', methods=['POST'])
def asr_recorder_delete_task():
    return jsonify({"text": "OK"}), 200

@bp.route('/v1/asr/task/create', methods=['POST'])
def asr_task_create():
    """
    Receive an audio file from the client and return the transcribed text.
    
    """
    file = request.files.get('file')
    file_content = file.read()
    size = len(file_content)

    # check
    if size > 512 * 1024 * 1024:
        return {'error_code': -2, 'error_desc': 'file size > 512m', 'data': {'task_id': ""}}
    
    hot_uid = ''
    sensitive_uid = ''
    if request.form.get('hot_uid', None):
        hot_uid = str(request.form['hot_uid'])

    if request.form.get('sensitive_uid', None):
        sensitive_uid = str(request.form['sensitive_uid'])

    unique_id = hot_uid + ";" + sensitive_uid

    priority = request.form['priority']
    if not all([file]):
        return {'error_code': -1, 'error_desc': 'params is null', 'data': {'task_id': ''}}
    logger.info("create task unique_id: " + str(unique_id) + " filename: " + str(file.filename))
    file_uuid = uuid.uuid1()
    file_path = "tmp/" + str(file_uuid) + "_" + str(file.filename)
    try:
        with open(file_path, "wb+") as f:
            f.write(file_content)
            f.close()
    except:
        logger.info("file error")
        return {'error_code': -3, 'error_desc': 'server can not create the file', 'data': {'task_id': ""}}

    # 采样率
    frame_rate = 0
    # 几通道
    channel = 2

    # 判断文件格式
    try:
        song = wave.open(file_path)
        frame_rate = song.getframerate()
        channel = song.getnchannels()
        is_type_ok = True
    except:
        is_type_ok = False

    if not is_type_ok:
        try:
            song = MP3(file_path)
            frame_rate = song.info.sample_rate
            channel = song.info.channels
            is_type_ok = True
        except:
            is_type_ok = False

    if not is_type_ok:
        return {'error_code': -2, 'error_desc': '必须是mp3或者wav', 'data': {'task_id': ""}}

    # 判断采样率
    if frame_rate not in (8000, 16000):
        return {'error_code': -2, 'error_desc': '采样率必须是8000或者16000HZ', 'data': {'task_id': ""}}
    # 判断几通道
    if channel != 1:
        return {'error_code': -2, 'error_desc': '必须是单通道', 'data': {'task_id': ""}}

    task_id = ScheduledTask.start_task(priority, unique_id, file_path)
    if task_id is not None:
        ScheduledTask.is_transmit = True
        return {'error_code': 0, 'error_desc': 'ok', 'data': {'task_id': task_id}}
    return {'error_code': -2, 'error_desc': 'create task fail', 'data': {'task_id': ""}}
@bp.route('/v1/asr/task/status', methods=['POST'])
def asr_task_status():
    return jsonify({"text": "OK"}), 200
@bp.route('/v1/asr/task/cancel', methods=['POST'])
def asr_task_cancel():
    return jsonify({"text": "OK"}), 200

@bp.route('/update_transmit', methods=['POST'])
def update_transmit():
    return jsonify({"text": "OK"}), 200

@bp.route('/api/v2/asr/lm/bases', methods=['GET'])
def get_lm_bases():
    # 系统启动时会自动尝试创建基础模型，可以通过下面的命令确认是否基础模型已经创建：
    # curl -X GET \
    #  --header 'Content-Type: multipart/form-data' \
    #  --header 'Accept: application/json' \
    #  --header 'X-NLS-Token: default' \
    #  'http://nls-slp.aliyun.test:8701/api/v2/asr/lm/bases'
    # 如果返回的结果里 total_items 的值不是 0，表示系统已经存在定制模
    j = {"total_items": 0}
    return jsonify(j), 200

@bp.route('/api/v1/asr/class-vocabs', methods=['POST'])
def add_class_vocab():
    # 5.4.定制类热词
    # 执行下面的命令添加类热词，这里我们添加了“北京”和“上海”这两个地名：（参考 7.6.1 节）
    # curl -X POST \
    # --header 'Content-Type: application/json' --header 'Accept: application/json' \
    # --header 'X-NLS-User: default' --header 'X-NLS-Token: default' \
    # -d '{
    # "id": "demo-vocab",
    # "name": "示例词表",
    # "description": "这是一个示例词表",
    # "words": ["北京", "上海"]
    # }' 'http://nls-slp.aliyun.test:8701/api/v1/asr/class-vocabs'


    # 7.6.1.创建词表
    # 路径：
    # POST /api/v1/asr/class-vocabs
    # 描述：
    # 创建一个定制类热词词表。

    # 输入参数：
    # 名称 位置 说明
    # vocab Body 类热词词表对象（AsrClassVocab）。可以设置如下参数：
    # id：词表 ID，可选，如果为空则自动生成；
    # name：词表名称；
    # description：词表描述信息，可选；
    # words：要添加的词，是一个字符串数组。

    # 请求示例：
    # curl -X POST \
    #  --header 'Content-Type: application/json' --header 'Accept: application/json' \
    #  --header 'X-NLS-User: default' --header 'X-NLS-Token: default' \
    #  -d '{
    #  "id": "demo-vocab",
    #  "name": "示例词表",
    #  "description": "这是一个示例词表",
    #  "words": ["苹果", "西瓜"]
    # }' 'http://nls-slp.aliyun.test:8701/api/v1/asr/class-vocabs'

    # 输出参数：
    # 名称 说明
    # class_vocab _id 词表 ID。

    # 响应示例：
    # {
    #  "request_id": "7130914d32a3441db06747523675d9ff",
    #  "class_vocab_id": "demo-vocab"
    # }

    j = {
            "request_id": "7130914d32a3441db06747523675d9ff",
            "class_vocab_id": "demo-vocab"
        }

    return jsonify(j), 200

@bp.route('/api/v1/asr/class-vocabs', methods=['GET'])
def get_class_vocabs():
    # 7.6.2.列举词表
    # 路径：
    # GET /api/v1/asr/class-vocabs
    # 描述：
    # 列举定制类热词词表。

    # 输入参数：
    # 名称 位置 说明
    # page_number Query 页号，从 1 开始编号。可选，默认值是 1。
    # page_size Query 页大小，范围在 1 到 100 之间。可选，默认是 10。

    # 请求示例：
    # curl -X GET --header 'Accept: application/json' \
    # --header 'X-NLS-User: default' --header 'X-NLS-Token: default' \
    # 'http://nls-slp.aliyun.test:8701/api/v1/asr/class-vocabs'

    # 输出参数：
    # 名称 说明
    # page 类热词词表对象（AsrClassVocab）的分页结果。

    # 响应示例：
    # {
    # "request_id": "7130914d32a3441db06747523675d9ff",
    # "page": {
    # "content": [
    # {
    # "id": "demo-vocab",
    # "name": "示例词表",
    # "description": "这是一个示例词表",
    # "size": 4,
    # "md5": "f1d3ff8443297732862df21dc4e57262",
    # "create_time": "2018-11-01 20:02:56",
    # "update_time": "2018-11-01 20:02:56"
    # }
    # ],
    # "total_pages": 1,
    # "total_items": 1,
    # "page_number": 1,
    # "page_size": 10
    # }
    # }

    j = {
        "request_id": "7130914d32a3441db06747523675d9ff",
        "page": {
            "content": [
                {
                    "id": "demo-vocab",
                    "name": "示例词表",
                    "description": "这是一个示例词表",
                    "size": 4,
                    "md5": "f1d3ff8443297732862df21dc4e57262",
                    "create_time": "2018-11-01 20:02:56",
                    "update_time": "2018-11-01 20:02:56"
                }
            ],
            "total_pages": 1,
            "total_items": 1,
            "page_number": 1,
            "page_size": 10
        }
    }

    return jsonify(j), 200

@bp.route('/api/v1/asr/class-vocabs/<vocab_id>', methods=['GET'])
def get_class_vocab(vocab_id):
    # 7.6.3.查询词表
    # 路径：
    # GET /api/v1/asr/class-vocabs/{vocab_id}
    # 描述：
    # 查询定制类热词词表。
    # 输入参数：
    # 名称 位置 说明
    # vocab_id Path 要查询的词表 ID。
    # 请求示例：
    # curl -X GET --header 'Accept: application/json' \
    #  --header 'X-NLS-User: default' --header 'X-NLS-Token: default' \
    #  'http://nls-slp.aliyun.test:8701/api/v1/asr/class-vocabs/demo-vocab'
    # 输出参数：
    # 名称 说明
    # class_vocab 类热词词表对象（AsrClassVocab）。
    # 响应示例：
    # {
    #  "request_id": "7130914d32a3441db06747523675d9ff",
    #  "class_vocab": {
    #  "id": "demo-vocab",
    #  "name": "示例词表（二）",
    #  "description": "这是另一个示例词表",
    #  "size": 4,
    #  "md5": "f1d3ff8443297732862df21dc4e57262",
    #  "words": [
    #  "苹果",
    #  "西瓜"
    #  ],
    #  "create_time": "2018-11-01 18:31:06",
    #  "update_time": "2018-11-01 18:31:06"
    #  }
    # }

    j = {
        "request_id": "7130914d32a3441db06747523675d9ff",
        "class_vocab": {
            "id": "demo-vocab",
            "name": "示例词表（二）",
            "description": "这是另一个示例词表",
            "size": 4,
            "md5": "f1d3ff8443297732862df21dc4e57262",
            "words": [
                "苹果",
                "西瓜"
            ],
            "create_time": "2018-11-01 18:31:06",
            "update_time": "2018-11-01 18:31:06"
        }
    }

    return jsonify(j), 200


@bp.route('/api/v1/asr/class-vocabs', methods=['PUT'])
def update_class_vocab():
    # 7.6.4.更新词表
    # 路径：
    # POST /api/v1/asr/class-vocabs
    # 描述：
    # 更新定制类热词词表。
    # 输入参数：
    # 名称 位置 说明
    # vocab Body 类热词词表对象（AsrClassVocab）。可以设置如下参数：
    # id：词表 ID，可选，如果为空则自动生成；
    # name：词表名称；
    # description：词表描述信息，可选；
    # words：要添加的词，是一个字符串数组，全量替换原有的词。
    # 请求示例：
    # curl -X PUT \
    #  --header 'Content-Type: application/json' --header 'Accept: application/json' \
    #  --header 'X-NLS-User: default' --header 'X-NLS-Token: default' -d '{
    #  "id": "demo-vocab",
    #  "name": "示例词表（二）",
    #  "description": "这是另一个示例词表",
    #  "words": ["苹果", "西瓜"]
    # }' 'http://nls-slp.aliyun.test:8701/api/v1/asr/class-vocabs'
    # 输出参数：
    # 名称 说明
    # request_id 请求 ID。
    # 响应示例：
    # {
    #  "request_id": "7130914d32a3441db06747523675d9ff"
    # }

    j = {"request_id": "7130914d32a3441db06747523675d9ff"}

    return jsonify(j), 200

@bp.route('/api/v1/asr/class-vocabs/<vocab_id>', methods=['DELETE'])
def delete_class_vocab(vocab_id):
    # 7.6.5.删除词表
    # 路径：
    # DELETE /api/v1/asr/class-vocabs/{vocab_id}
    # 描述：
    # 删除定制类热词词表。
    # 输入参数：
    # 名称 位置 说明
    # vocab_id Path 要删除的词表 ID。
    # 请求示例：
    # curl -X DELETE --header 'Accept: application/json' \
    #  --header 'X-NLS-User: default' --header 'X-NLS-Token: default' \
    #  'http://nls-slp.aliyun.test:8701/api/v1/asr/class-vocabs/demo-vocab'
    # 输出参数：
    # 名称 说明
    # request_id 请求 ID。
    # 响应示例：
    # {
    #  "request_id": "7130914d32a3441db06747523675d9ff"
    # }
    j = {"request_id": "7130914d32a3441db06747523675d9ff"}

    return jsonify(j), 200

@bp.route('/api/v1/asr/vocabs', methods=['POST'])
def add_vocab(vocab_id):
    # 7.7.1.创建词表
    # 路径：
    # POST /api/v1/asr/vocabs
    # 输入参数：
    # 名称 位置 说明
    # vocab Body 泛热词词表对象（AsrVocab）。可以设置如下参数：
    # id：词表 ID，可选，如果为空则自动生成；
    # name：词表名称；
    # description：词表描述信息，可选；
    # word_weights：要添加的词，是一个词典，键为词，值为权重。
    # 请求示例：
    # curl -X POST \
    # --header 'Content-Type: application/json' \
    # --header 'Accept: application/json' \
    # --header 'X-NLS-User: default' \
    # --header 'X-NLS-Token: default' \
    # -d '{
    # "id": "demo-vocab",
    # "name": "示例词表",
    # "description": "这是一个示例词表",
    # "word_weights": {"苹果": 2, "西瓜": 3}
    # }' 'http://nls-slp.aliyun.test:8701/api/v1/asr/vocabs'
    # 输出参数：
    # 名称 说明
    # vocab _id 词表 ID。
    # 响应示例：
    # {
    # "request_id": "7130914d32a3441db06747523675d9ff",
    # "vocab_id": "demo-vocab"
    # }

    j = {"request_id": "7130914d32a3441db06747523675d9ff", "vocab_id": "demo-vocab"}

    return jsonify(j), 200

@bp.route('/api/v1/asr/vocabs', methods=['GET'])
def get_vocabs():
    # 7.7.2.列举词表
    # 路径：
    # GET /api/v1/asr/vocabs
    # 输入参数：
    # 名称 位置 说明
    # page_number Query 页号，从 1 开始编号。可选，默认值是 1。
    # page_size Query 页大小，范围在 1 到 100 之间。可选，默认是 10。
    # 请求示例：
    # curl -X GET \
    # --header 'Accept: application/json' \
    # --header 'X-NLS-User: default' \
    # --header 'X-NLS-Token: default' \
    # 'http://nls-slp.aliyun.test:8701/api/v1/asr/vocabs'
    # 输出参数：
    # 名称 说明
    # request_id 请求 ID。
    # page 泛热词词表对象（AsrVocab）的分页结果。
    # 响应示例：
    # {
    #  "request_id": "7130914d32a3441db06747523675d9ff",
    #  "page": {
    #  "content": [
    #  {
    #  "id": "demo-vocab",
    #  "name": "示例词表",
    #  "description": "这是一个示例词表",
    #  "size": 4,
    #  "md5": "f1d3ff8443297732862df21dc4e57262",
    #  "create_time": "2018-11-01 20:11:42",
    #  "update_time": "2018-11-01 20:11:42"
    #  }
    #  ],
    #  "total_pages": 1,
    #  "total_items": 1,
    #  "page_number": 1,
    #  "page_size": 10
    #  }
    #  }
    j = {
        "request_id": "7130914d32a3441db06747523675d9ff", 
        "page": 
            {
                "total_pages": 1,
                "total_items": 1,
                "page_number": 1,
                "page_size": 10,

                "content": 
                    [
                        {"id": "demo-vocab", "name": "示例词表", "description": "这是一个示例词表", "size": 4, "md5": "f1d3ff8443297732862df21dc4e57262", "create_time": "2018-11-01 20:11:42", "update_time": "2018-11-01 20:11:42"}
                    ]
            }
        }
    
    return jsonify(j), 200

@bp.route('/api/v1/asr/vocabs/<vocab_id>', methods=['GET'])
def get_vocab(vocab_id):
    # 7.7.3.查询词表
    # 路径：
    # GET /api/v1/asr/vocabs/{vocab_id}
    # 输入参数：
    # 名称 位置 说明
    # vocab_id Path 要查询的词表 ID。
    # 请求示例：
    # curl -X GET \
    #  --header 'Accept: application/json' \
    #  --header 'X-NLS-User: default' \
    #  --header 'X-NLS-Token: default' \
    # 'http://nls-slp.aliyun.test:8701/api/v1/asr/vocabs/demo-vocab'
    # 输出参数：
    # 名称 说明
    # vocab 泛热词词表对象（AsrVocab）。
    # 响应示例：
    # {
    #  "request_id": "7130914d32a3441db06747523675d9ff",
    #  "vocab": {
    #  "id": "demo-vocab",
    #  "name": "示例词表",
    #  "description": "这是一个示例词表",
    #  "size": 4,
    #  "word_weights": {
    #  "苹果": 2,
    #  "西瓜": 3
    #  }
    #  }
    #  }
    j = {
        "request_id": "7130914d32a3441db06747523675d9ff", 
        "vocab": 
            {
                "id": "demo-vocab",
                "name": "示例词表",
                "description": "这是一个示例词表",
                "size": 4,
                "md5": "f1d3ff8443297732862df21dc4e57262",
                "create_time": "2018-11-01 20:11:42", 
                "md5": "f1d3ff8443297732862df21dc4e57262",
                "create_time": "2018-11-01 20:11:42",
                "update_time": "2018-11-01 20:11:42",
                "word_weights": {
                "苹果": 2,
                "西瓜": 3
                }
            }
        }

    return jsonify(j), 200

@bp.route('/api/v1/asr/vocabs/<vocab_id>', methods=['PUT'])
def update_vocab(vocab_id):
    # 7.7.4.更新词表
    # 路径：
    # POST /api/v1/asr/vocabs
    # 输入参数：
    # 智能语音 V2.X 【公开】 
    # 文档版本：20210430 XCVII 
    # 名称 位置 说明
    # vocab Body 类热词词表对象（AsrClassVocab）。可以设置如下参数：
    # id：词表 ID，可选，如果为空则自动生成；
    # name：词表名称；
    # description：词表描述信息，可选；
    # word_weights：要添加的词，是一个词典，键为词，值为权重。全量替换原有的词。
    # 请求示例：
    # curl -X PUT \
    #  --header 'Content-Type: application/json' \
    #  --header 'Accept: application/json' \
    #  --header 'X-NLS-User: default' \
    #  --header 'X-NLS-Token: default' \
    #  -d '{
    #  "id": "demo-vocab",
    #  "name": "示例词表（二）",
    # "description": "这是另一个示例词表",
    #  "word_weights": {"苹果": 2, "西瓜": 3}
    #  }' 'http://nls-slp.aliyun.test:8701/api/v1/asr/vocabs'
    j = request.get_json()
    print(j)
    return jsonify(j), 200


@bp.route('/api/v1/asr/vocabs/<vocab_id>', methods=['DELETE'])
def delete_vocab(vocab_id):
    # 7.7.5.删除词表
    # 路径：
    # DELETE /api/v1/asr/vocabs/{vocab_id}
    # 输入参数：
    # 名称 位置 说明
    # vocab_id Path 要删除的词表 ID。
    # 请求示例：
    # curl -X DELETE \
    #  --header 'Accept: application/json' \
    #  --header 'X-NLS-User: default' \
    #  --header 'X-NLS-Token: default' \
    #  'http://nls-slp.aliyun.test:8701/api/v1/asr/vocabs/demo-vocab'
    # 输出参数：
    # 名称 说明
    # request_id 请求 ID。
    # 响应示例：
    # {
    #  "request_id": "7130914d32a3441db06747523675d9ff"
    #  }
    j = {
        "request_id": "7130914d32a3441db06747523675d9ff"
    }

    return jsonify(j), 200


import os
from modelscope.pipelines import pipeline
from modelscope.utils.constant import Tasks
import torch
import torchaudio
import numpy as np

def extract_embedding(audio_path, model_name='damo/speech_campplus_sv_zh-cn_16k-common'):
    """提取音频文件的声纹特征向量"""
    # 加载声纹提取模型
    sv_pipeline = pipeline(
        task=Tasks.speaker_verification,
        model=model_name
    )
    
    # 提取特征
    result = sv_pipeline(audio_path)
    
    # 返回特征向量
    return result['embedding']

def calculate_similarity(embedding1, embedding2):
    """计算两个声纹特征向量的余弦相似度"""
    # 将特征向量转换为numpy数组
    embedding1 = np.array(embedding1)
    embedding2 = np.array(embedding2)
    
    # 计算余弦相似度
    similarity = np.dot(embedding1, embedding2) / (np.linalg.norm(embedding1) * np.linalg.norm(embedding2))
    
    return similarity

def verify_speaker(reference_audio, test_audio, threshold=0.7, model_name='damo/speech_campplus_sv_zh-cn_16k-common'):
    """验证两个音频是否来自同一个说话人"""
    # 提取两个音频的声纹特征
    ref_embedding = extract_embedding(reference_audio, model_name)
    test_embedding = extract_embedding(test_audio, model_name)
    
    # 计算相似度
    similarity = calculate_similarity(ref_embedding, test_embedding)
    
    # 判断是否为同一说话人
    is_same_speaker = similarity >= threshold
    
    return {
        'is_same_speaker': is_same_speaker,
        'similarity': similarity,
        'threshold': threshold
    }

def batch_identify_speakers(reference_dir, test_audio, threshold=0.7, model_name='damo/speech_campplus_sv_zh-cn_16k-common'):
    """在多个参考说话人中识别测试音频的说话人"""
    # 获取参考目录中的所有音频文件
    reference_files = [f for f in os.listdir(reference_dir) if f.endswith(('.wav', '.mp3'))]
    
    results = []
    
    for ref_file in reference_files:
        ref_path = os.path.join(reference_dir, ref_file)
        print(f"ref_file={ref_path}, test_audio={test_audio}")
        result = verify_speaker(ref_path, test_audio, threshold, model_name)
        result['reference_file'] = ref_file
        results.append(result)
    
    # 按相似度排序
    results.sort(key=lambda x: x['similarity'], reverse=True)
    
    return results

# 使用示例
if __name__ == "__main__":
    # 选择模型：Cam++ 或 ERes2Net
    # Cam++模型
    model_name = 'damo/speech_campplus_sv_zh-cn_16k-common'
    
    # ERes2Net模型
    # model_name = 'damo/speech_eres2net_sv_zh-cn_16k-common'
    
    # 验证两个音频是否来自同一说话人
    reference_audio = "sv_references/jacky-16k-16bits-mono.wav" # 'reference_audio.wav'
    test_audio = 'jacky.wav' # 'test_audio.wav'
    
    verification_result = verify_speaker(reference_audio, test_audio, model_name=model_name)
    print(f"声纹验证结果: {'是同一说话人' if verification_result['is_same_speaker'] else '不是同一说话人'}")
    print(f"相似度得分: {verification_result['similarity']:.4f}")
    print(f"阈值: {verification_result['threshold']}")
    
    # 批量识别说话人
    reference_dir = 'sv_references'
    batch_results = batch_identify_speakers(reference_dir, test_audio, model_name=model_name)
    
    print("\n批量识别结果:")
    for i, result in enumerate(batch_results):
        print(f"{i+1}. {result['reference_file']}: 相似度 {result['similarity']:.4f}, {'匹配' if result['is_same_speaker'] else '不匹配'}")



# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: main_server.py 
@info: 消息模版
"""
import argparse
import threading
import asyncio
import signal
import sys
from time import sleep
from werkzeug.serving import run_simple

from oddasr.odd_asr_app import app
from oddasr.logic.odd_asr_instance import init_instance_file, init_instance_sentence
from oddasr.logic.odd_wss_server import init_instances_stream, start_wss_server
from oddasr.logic.scheduled_task import ScheduledTask
from oddasr.log import logger
from oddasr import odd_asr_config as config

# 全局变量用于控制程序退出
shutdown_flag = threading.Event()

def signal_handler(sig, frame):
    """信号处理函数，用于优雅地停止程序"""
    logger.info("接收到退出信号，正在停止所有服务...")
    shutdown_flag.set()
    
    # 停止Flask服务器（通过设置shutdown_flag，run_simple会自动停止）
    
    # 等待所有线程完成
    if 'scheduled_task' in globals():
        scheduled_task.thread_run = False
        scheduled_task.join(timeout=5)
    
    logger.info("所有服务已停止，程序退出")
    sys.exit(0)

def main():
    # 注册信号处理
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)
    
    def start_wss_in_thread():
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            # 创建一个任务来运行WebSocket服务器
            server_task = loop.create_task(start_wss_server())
            
            # 等待shutdown_flag被设置
            while not shutdown_flag.is_set():
                loop.run_until_complete(asyncio.sleep(0.5))
            
            # 取消WebSocket服务器任务
            server_task.cancel()
            try:
                loop.run_until_complete(server_task)
            except asyncio.CancelledError:
                logger.info("WebSocket server task cancelled")
        finally:
            loop.close()

    # 初始化WebSocket实例
    if not config.odd_asr_cfg["disable_stream"]:
        init_instances_stream()
    
    # start websocket server
    if not config.odd_asr_cfg["disable_stream"]:
        wss_thread = threading.Thread(target=start_wss_in_thread)
        wss_thread.daemon = True  # 设置为守护线程，主线程退出时自动退出
        wss_thread.start()
        logger.info("WebSocket server started.")
    else:
        logger.info("WebSocket server disabled.")

    # init file/sentence ASR instances
    init_instance_file()
    init_instance_sentence()
    logger.info("File ASR and sentence ASR instances started.")

    # Start scheduled task thread
    global scheduled_task
    scheduled_task = ScheduledTask(status_notifier=None)
    scheduled_task.start()
    logger.info("Scheduled task thread started.")

    # Start Flask server with HTTPS support
    logger.info(f"Starting server on {'https' if config.odd_asr_cfg['enable_https'] else 'http'}://{config.HOST}:{config.PORT}")
    
    # 使用Werkzeug的run_simple替代app.run()，这样可以更好地控制服务器
    ssl_context = None
    if config.odd_asr_cfg["enable_https"]:
        ssl_context = (config.odd_asr_cfg["ssl_cert_path"], config.odd_asr_cfg["ssl_key_path"])
    
    # 运行Flask服务器，直到shutdown_flag被设置
    run_simple(
        hostname=config.HOST,
        port=config.PORT,
        application=app,
        use_reloader=False,  # 禁用自动重载
        use_debugger=config.Debug,
        ssl_context=ssl_context
    )



from sqlalchemy import or_, desc, func

from oddasr.log import logger
from oddasr.model.db import Session
from oddasr.model.meeting import CMeeting, CMeetingStatus2
from oddasr.model.text import CText

# apply res suc.
class CStorageMinutes:

    @classmethod
    def applyMeetingRes(self, strSessionId: str, strMeetingE164: str, strMeetingAlias: str, strBeginTime: str,
                        bIsOpenMinutes: bool):
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.session_id == strSessionId).first()
        if meeting == None:
            meeting = CMeeting()
            meeting.session_id = strSessionId
            meeting.meeting_begin_time = strBeginTime
            meeting.meeting_alias = strMeetingAlias
            meeting.meeting_type = 2
            meeting.meeting_status = CMeetingStatus2.MEETING_OPEN_STATUS_INIT
            session.add(meeting)
            session.commit()
            session.close()
        else:
            if bIsOpenMinutes and meeting.meeting_status == CMeetingStatus2.MEETING_OPEN_STATUS_INIT:
                meeting.meeting_status = CMeetingStatus2.MEETING_OPEN_STATUS_ENCODE
                session.commit()
            session.close()

    @classmethod
    def openMeetingMinutes(self, strSessionId: str):
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.session_id == strSessionId).first()
        if meeting != None and meeting.meeting_status == CMeetingStatus2.MEETING_OPEN_STATUS_ENCODE:
            meeting.meeting_status = CMeetingStatus2.MEETING_OPEN_STATUS_ENCODE
            session.commit()
        session.close()

    @classmethod
    def getAllMeetingsByKey(self, index, num, key=None):
        session = Session()
        count = 0
        qc = session.query(func.count(CMeeting.session_id))
        q = session.query(CMeeting)

        if key != None and key != "":
            qc = qc.filter(or_(CMeeting.meeting_alias.like('%' + key + '%'),
                               CMeeting.meeting_addr.like('%' + key + '%'),
                               CMeeting.meeting_participant.like('%' + key + '%'),
                               CMeeting.session_id == key))
            qc = qc.filter(CMeeting.meeting_status >= 0)
            q = q.filter(or_(CMeeting.meeting_alias.like('%' + key + '%'),
                             CMeeting.meeting_addr.like('%' + key + '%'),
                             CMeeting.meeting_participant.like('%' + key + '%'),
                             CMeeting.session_id == key))
            q = q.filter(CMeeting.meeting_status >= 0)

        q = q.order_by(desc(CMeeting.id))
        count = qc.scalar()
        list = q.limit(num).offset((index) * num).all()
        for meeting in list:
            if meeting.meeting_type == 1:
                split_strs = meeting.meeting_alias.split('/')
                meeting.meeting_alias = split_strs[-1]
                # 查找第一个_
                pos = meeting.meeting_alias.find('_')
                meeting.meeting_alias = meeting.meeting_alias[pos + 1:]
        session.close()
        return count, list

    @classmethod
    def addMeetingText(self, strMoid: str, strTermE164: str,
                       strTermAlias: str, nBg: int, nEd: int, 
                       strText: str, nFlag: int, nTime: int , nSign:int):
        session = Session()
        text = CText()
        text.session_id = strMoid
        text.text_term_e164 = strTermE164
        text.text_term_alias = strTermAlias
        text.text_bg_time = nBg
        text.text_ed_time = nEd
        text.text_content = strText
        text.text_time = nTime
        text.text_flag = nFlag
        text.text_sign = nSign
        session.add(text)
        session.commit()
        session.close()
    
    @classmethod
    def opMeetingText(self, strMoid: str,  strTermE164: str,
                       strTermAlias: str, nBg: int, nEd: int, 
                       strText: str,  nTime: int , nMixBg:int , nMixEd:int):
        session = Session()
        session.query(CText).filter(CText.session_id == strMoid,
                                              CText.text_bg_time >= nMixBg, 
                                              CText.text_sign == 1 ,
                                              CText.text_ed_time <= nMixEd).update({CText.text_sign:2})
        
        text = CText()
        text.session_id = strMoid
        text.text_term_e164 = strTermE164
        text.text_term_alias = strTermAlias
        text.text_bg_time = nBg
        text.text_ed_time = nEd
        text.text_content = strText
        text.text_time = nTime
        text.text_flag = 0
        text.text_sign = 3
        session.add(text)
        session.commit()   
        session.close()   

    @classmethod
    def updateMeetingText(self, strId: str, strText: str):
        logger.info(f"strId:{strId},strText:{strText}")
        session = Session()
        meeting = session.query(CText).filter(CText.text_id == strId).first()
        if meeting != None:
            meeting.text_content = strText
            session.commit()
        session.close()

    @classmethod
    def getMeetingInfoById(self, meeting_id: int):
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.id == meeting_id).first()

        return meeting

    @classmethod
    def getUnEncodeMeetingMoids(self):
        session = Session()
        moids = session.query(CMeeting.session_id).filter(CMeeting.meeting_status == CMeetingStatus2.MEETING_OPEN_STATUS_ENCODE,
                                                            CMeeting.meeting_type == 2).all()
        session.close()
        return moids

    @classmethod
    def delMeetingInfoById(self, meeting_id: int):
        session = Session()
        session.query(CMeeting).filter(CMeeting.id.in_([meeting_id])).delete(synchronize_session=False)
        session.commit()
        session.close()

    @classmethod
    def getMeetingInfo(self, strMoid: str):
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.session_id == strMoid).first()
        count = session.query(func.count(CText.text_id)).filter(CText.session_id == strMoid).scalar()
        session.close()
        return meeting, count

    @classmethod
    def getMeetingText(self, strMoid: str, nPage: int, nNum: int , bOP:bool):
        session = Session()
        if bOP:
            qc = session.query(func.count(CText.text_id)).filter(CText.session_id == strMoid).filter(or_(CText.text_sign == 0 , 
            CText.text_sign == 1 , CText.text_sign == 3))
            q = session.query(CText).filter(CText.session_id == strMoid).filter(or_(CText.text_sign == 0 , 
            CText.text_sign == 1 , CText.text_sign == 3))
        else:
            qc = session.query(func.count(CText.text_id)).filter(CText.session_id == strMoid).filter(or_(CText.text_sign == 0 , 
            CText.text_sign == 1 , CText.text_sign == 2))
            q = session.query(CText).filter(CText.session_id == strMoid).filter(or_(CText.text_sign == 0 , 
            CText.text_sign == 1 , CText.text_sign == 2))
        q = q.order_by(CText.text_bg_time)
        count = qc.scalar()
        if nPage <= 0 or nNum <= 0:
            list = q.all()
        else:
            list = q.limit(nNum).offset((nPage - 1) * nNum).all()
        session.close()
        return list, count

    @classmethod
    def getMeetingAllText(self, strMoid: str, nPage: int, nNum: int, sign: int):
        session = Session()
        qc = session.query(func.count(CText.text_id)).filter(CText.session_id == strMoid)
        if sign != -1:
            q = session.query(CText).filter(CText.session_id == strMoid).filter(or_(CText.text_sign == sign))
        else:
            q = session.query(CText).filter(CText.session_id == strMoid)
        q = q.order_by(CText.text_bg_time)
        count = qc.scalar()
        if nPage <= 0 or nNum <= 0:
            list = q.all()
        else:
            list = q.limit(nNum).offset((nPage - 1) * nNum).all()
        session.close()
        return list, count

    @classmethod
    def addOpenMeeting(self, strSessionId: str, strMeetingName: str, strMeetingAddr: str
                       , strMeetingParticipant: str, strBeginTime: str):
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.session_id == strSessionId).first()
        if meeting != None:
            session.close()
            return False

        meeting = CMeeting()
        meeting.session_id = strSessionId
        meeting.meeting_participant = strMeetingParticipant
        meeting.meeting_begin_time = strBeginTime
        meeting.meeting_alias = strMeetingName
        meeting.meeting_addr = strMeetingAddr
        meeting.meeting_type = 0
        meeting.meeting_status = CMeetingStatus2.MEETING_OPEN_STATUS_INIT
        session.add(meeting)
        session.commit()
        session.close()
        return True

    @classmethod
    def updateStatus(self, strSessionId: str, strEndTime: str, nStatus: int):
        """
        更新会议状态
        :param strSessionId: 会议唯一标识
        :param strEndTime: 结束时间
        :param nStatus: 会议状态
        :return: True/False
        """
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.session_id == strSessionId).first()
        if meeting == None:
            session.close()
            return False

        meeting.meeting_status = nStatus
        if strEndTime != None:
            meeting.meeting_end_time = strEndTime
        session.commit()
        session.close()
        return True

    @classmethod
    def addOpenFileMeeting(self, strSessionId: str, strRank: str, strUniqueId: str, strFileName: str, strBeginTime: str):
        session = Session()
        strSessionId = str(strSessionId)
        meeting = session.query(CMeeting).filter(CMeeting.session_id == strSessionId).first()
        if meeting != None:
            session.close()
            return False

        meeting = CMeeting()
        meeting.session_id = strSessionId
        # meeting.meeting_e164 = strRank
        meeting.meeting_begin_time = strBeginTime
        meeting.meeting_alias = strFileName
        meeting.meeting_participant = strUniqueId
        meeting.meeting_type = 1
        meeting.meeting_status = CMeetingStatus2.MEETING_OPEN_STATUS_INIT
        session.add(meeting)
        session.commit()
        session.close()
        return True

    #
    # arrText:{'session_id':strUuid,'text_bg_time':,'text_ed_time':,'text_content':,}
    #
    @classmethod
    def updateOpenFileMeetingEnd(self, strSessionId: str, strEndTime: str, arrTexts: list):
        """
        更新打开文件会议结束时间
        :param strSessionId: 会议唯一标识
        :param strEndTime: 结束时间
        :param arrTexts: 会议文本列表
        :return:
        """
        logger.debug(f"updateOpenFileMeetingEnd strSessionId:{strSessionId} strEndTime:{strEndTime} arrTexts:{arrTexts}")
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.session_id == strSessionId).first()
        if meeting == None:
            session.close()
            return False
        meeting.meeting_status = CMeetingStatus2.MEETING_OPEN_STATUS_FINISH
        meeting.meeting_end_time = strEndTime
        if arrTexts != None and len(arrTexts) > 0:
            session.execute(CText.__table__.insert(), arrTexts)
        session.commit()
        session.close()
        return True

    @classmethod
    def getOpenFileMeetingInfo(self, strSessionId: str):
        meeting, count = self.getMeetingInfo(strSessionId)
        if meeting != None:
            return {'uuid': meeting.session_id, 'aliTaskId': meeting.meeting_addr,
                    'uniqueId': meeting.meeting_participant, "status": meeting.meeting_status,
                    'priority': '',  # meeting.meeting_e164,
                    'fileName': meeting.meeting_alias, 'begin_time': meeting.meeting_begin_time,
                    "end_time": meeting.meeting_end_time}
        return None

    @classmethod
    def getOpenFileMeetingInfoByPriority(self):
        session = Session()
        meeting = session.query(CMeeting).filter(CMeeting.meeting_type == 1,
                                                 CMeeting.meeting_status == CMeetingStatus2.MEETING_OPEN_STATUS_INIT).order_by(
            CMeeting.session_id.desc()).first()
        session.close()
        if meeting != None:
            return {'uuid': meeting.session_id, 
                    'aliTaskId': meeting.meeting_addr,
                    'uniqueId': meeting.meeting_participant, 
                    "status": meeting.meeting_status,
                    'priority': '', # meeting.meeting_e164,
                    'fileName': meeting.meeting_alias, 
                    'begin_time': meeting.meeting_begin_time,
                    "end_time": meeting.meeting_end_time}
        return None

    @classmethod
    def deleteOpenFileMeeting(self, strUuid: str):
        session = Session()
        results = session.query(CMeeting).filter(CMeeting.session_id == strUuid).all()
        for result in results:
            session.delete(result)
            session.commit()
        return True

    @classmethod
    def getMeetingConf(self, meeting_id: int):
        meeting = CStorageMinutes.getMeetingInfoById(meeting_id)
        content = self.getMeetingText(meeting.session_id, 0, 0, False)
        return meeting, content



import json

from flask import Blueprint, render_template, request, session, redirect, send_from_directory, send_file
from werkzeug.utils import secure_filename

import oddasr.odd_asr_config as config
from oddasr.log import logger

from oddasr.router.oddasr_session import session_required
from oddasr.logic import hotwords, sensitivewords, users

bp = Blueprint('front_asr', __name__, url_prefix='')

@bp.route('/asr_live.html')
def asr_live():
    return render_template('asr_live.html', servercfg=config.asr, username=session["user"])

@bp.route('/asr_file.html')
def asr_file():
    return render_template('asr_file.html', servercfg=config.asr)

@bp.route('/asr_sentence.html')
def asr_sentence():
    return render_template('asr_sentence.html', servercfg=config.asr)

@bp.route('/login', methods=['POST'])
def login():
    try:
        j = request.get_json()
        if users.check_user(j['user'], j['pwd']):
            session.permanent = True
            session['user'] = j['user']
            return {'code': 0}
        else:
            return {'code': -1}
    except:
        return {'code': -2}

@bp.route('/logout')
@session_required
def logout():
    session['user'] = None
    return redirect('/login.html', code=302)

@bp.route('/')
@session_required
def root():
    return redirect("index.html", code=302)

@bp.route('/index.html')
@session_required
def index():
    return render_template('index.html', username=session["user"])

@bp.route('/user_pwd.html')
@session_required
def user_pwd():
    return render_template('user_pwd.html', username=session['user'])

########################################
## template --> settings
########################################
@bp.route('/settings_others.html')
@session_required
def settings_others():
    data = {}
    return render_template('settings_others.html', data=data)

@bp.route('/slp_language_model.html')
@session_required
def slp_language_model():
    data = {}
    return render_template('slp_language_model.html', data=data)

@bp.route('/slp_acoustic_model.html')
@session_required
def slp_acoustic_model():
    data = {}
    return render_template('slp_acoustic_model.html', data=data)

@bp.route('/slp_textual_substitution.html')
@session_required
def slp_textual_substitution():
    data = {}
    return render_template('slp_textual_substitution.html', data=data)


@bp.route('/dialectmodal.html')
@session_required
def settings_langmodal():
    current_model = "" # check_current_model()
    model_list = {} # get_exist_model_packages()
    return render_template('dialectmodal.html', model=current_model, model_list=model_list)

@bp.route('/hotwords.html')
@session_required
def hotwords():
    res = {}

    # hot_words = []
    # data = hotwords.HotWordManage.get_hot_word_for_opensdk("test123", 1)
    # if not data.get("words", []):
    #     res = {"hotwords": hot_words}
    # else:
    #     hot_words = data.get("words")
    #     res = {"hotwords": eval(hot_words)}
    print(res)
    return render_template('hotwords.html', words=json.dumps(res))


@bp.route('/settings_hotwords_show.html')
@session_required
def hotwords_show():
    result = []
    # con = pymysql.connect(host=config.Mysql_Host, user=config.Mysql_User, password=config.Mysql_Pwd, charset='utf8mb4',
    #                       db='jdl')
    # cur = con.cursor()
    # cur.execute("select * from asrmanager_hotwords;")
    # data = cur.fetchall()
    # for i in data:
    #     words = eval(i[2])
    #     real_words = ""
    #     for j in words:
    #         if not real_words:
    #             real_words = j
    #         else:
    #             real_words = real_words + "," + j
    #     test = {"id": i[0], "unique_id": i[1].split('_')[0], "type": i[1].split('_')[1], "words": real_words}
    #     result.append(test)
    res = {"list": result}
    print(json.dumps(res))
    return render_template('settings_hotwords_show.html', words_list=result)


@bp.route('/sensiwords.html')
@session_required
def sensiwords():
    sensitivewords = {} # get_sensitive_words('test123')
    return render_template('sensiwords.html', data=sensitivewords)

@bp.route('/sensiwords_show.html')
@session_required
def sensiwords_show():
    result = []
    # con = pymysql.connect(host=config.Mysql_Host, user=config.Mysql_User, password=config.Mysql_Pwd, charset='utf8mb4',
    #                       db='jdl')
    # cur = con.cursor()
    # cur.execute("select * from asrmanager_sensitive_words;")
    # data = cur.fetchall()
    # for i in data:
    #     words = eval(i[2])
    #     real_words = ""
    #     for j in words:
    #         if not real_words:
    #             real_words = j
    #         else:
    #             real_words = real_words + "," + j
    #     test = {"id": i[0], "unique_id": i[1], "words": real_words}
    #     result.append(test)
    return render_template('sensiwords_show.html', words_list=result)


@bp.route('/sensiwords')
@session_required
def sensiwords2():
    res = {} # get_sensitive_words('test123')
    logger.info(res)
    return res



import requests

def test_file(audio_path: str, output_format: str = "txt"):
    # 设置服务的 URL
    url = "http://127.0.0.1:9002/v1/asr"
    # 定义 hotwords
    hotwords = "小落 小落同学 奥德元 小奥"
    # 打开音频文件
    with open(audio_path, "rb") as audio_file:
        # 发送 POST 请求
        response = requests.post(url, files={"audio": audio_file}, data={"hotwords": hotwords, "mode": "file", "output_format": output_format})
        # 输出结果
        if response.status_code == 200:
            try:
                print("Recognition Result:", response.json()["text"])
            except ValueError:
                print("Non-JSON response:", response.text)  # Print the raw response
        else:
            print("Error:", response.text)  # Print the raw error message


if __name__ == "__main__":
    import argparse
    import os

    parser = argparse.ArgumentParser(description="Your WAV file need to recoginze to text.")
    parser.add_argument("audio_path", type=str, help="file path of your input WAV.")
    parser.add_argument("output_format", type=str, help="output format, support: txt, spk or srt.")
    args = parser.parse_args()

    # test command:  python testAPI VCS-20200916175424.wav spk
    file = args.audio_path
    print(f"Current working directory: {os.getcwd()}")
    print(f"Full file path: {os.path.abspath(file)}")
    if not os.path.exists(file):
        print(f"File not found: {file}")
        exit(1) 

    fmt = args.output_format
    if fmt not in ["txt", "spk", "srt"]:
        print(f"output_format must be txt, spk or srt: {fmt}")
        exit(1)

    test_file(file, fmt)



import sys
sys.path.append('../')
from logic.minutes import CStorageMinutes
import uuid
import time

if __name__ == '__main__':
    
    ## add 
    strUuidId = str(uuid.uuid1())
    strRank = "0"
    strUniqueId = "123456"
    strFileName = "test.mp3"
    strBeginTime  = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    print(CStorageMinutes.addOpenFileMeeting(strUuidId , strRank , strUniqueId,strFileName,strBeginTime))

    print(CStorageMinutes.getOpenFileMeetingInfo(strUuidId))
    
    
    ## start
    meeting = CStorageMinutes.getOpenFileMeetingInfoByPriority()
    print(meeting)
    strAliTaskId = ''
    print(CStorageMinutes.updateOpenFileMeetingStart(meeting['uuid'] , strAliTaskId))

    print(CStorageMinutes.getOpenFileMeetingInfo(strUuidId))
    
    ## finish
    strEndTime  = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    texts = []
    item1 = {'session_id':meeting['uuid'],'text_bg_time':123,'text_ed_time':456 , 'text_content':'item1'}
    texts.append(item1)
    item2 =  {'session_id':meeting['uuid'],'text_bg_time':123,'text_ed_time':456 , 'text_content':'item2'}
    texts.append(item2)

    print(CStorageMinutes.updateOpenFileMeetingEnd(meeting['uuid'],strEndTime , texts))

    print(CStorageMinutes.getOpenFileMeetingInfo(strUuidId))
    ## if failed test
    
    strUuidId = str(uuid.uuid1())
    strRank = "0"
    strUniqueId = "123456"
    strFileName = "test.mp3"
    strBeginTime  = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    print(CStorageMinutes.addOpenFileMeeting(strUuidId , strRank , strUniqueId,strFileName,strBeginTime))

    
    meeting = CStorageMinutes.getOpenFileMeetingInfoByPriority()
    strAliTaskId = ''
    print(CStorageMinutes.updateOpenFileMeetingStart(meeting['uuid'] , strAliTaskId))
    print(meeting)

    #failed
    print(CStorageMinutes.updateOpenFileMeetingByRevert(meeting['uuid']))
    print(CStorageMinutes.getOpenFileMeetingInfoByPriority())


# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: odd_asr_stream.py 
@info: 消息模版
"""

import torch
import numpy as np
import queue
import threading
import time
import re
import copy
import os
from funasr import AutoModel

from oddasr.log import logger
from oddasr.logic.odd_asr_result import enque_asr_result, OddAsrStreamResult
import oddasr.odd_asr_config as config

class AudioFrame:
    def __init__(self, data, sr: int = 16000, channel=1, bit_depth=16, timestamp = 0):
        self.data = copy.deepcopy(data)             # 音频数据
        self.frame_timestamp = timestamp            # 从启动转写线程时间开始计时的时间戳
        self.sr = sr                                # 采样率
        self.channel = channel                      # 声道数
        self.bit_depth = bit_depth                  # 位深度
    def reset(self):
        self.data = np.array([], dtype=np.float64)
        self.frame_timestamp = 0
        self.sr = 16000
        self.channel = 1
        self.bit_depth = 16

class OddAsrStats:
    def __init__(self):
        self.reset()

    def reset(self):
        self.index = 0
        self.start_time = 0                         # 启动转写线程时间
        self.end_time = 0                           # 转写结束时间
        self.total_audio_recv_len = 0               # 接收音频总长度
        self.total_audio_input_len = 0              # 输入音频总长度
        self.total_asr_len = 0                      # 转写总长度
        self.total_asr_time = 0                     # 转写总时间
        self.last_recv_timestamp: float = 0.0       # 最后一次接收音频的时间戳
        self.last_reply_par_timestamp: float = 0.0  # 最后一次回复中间结果的时间戳
        self.last_reply_fin_timestamp: float = 0.0  # 最后一次生成最终结果的时间戳


class OddAsrParamsStream:
    _mode: str = "stream"
    _hotwords: str = "oddmeta xiaoluo"
    _rec_file:str =""
    _result_callback = None
    _result_callback: bool = False
    _is_final : bool = False
    _sentence_timestamp: bool = False

    _chunk_size:list = [0, 10, 5]
    _encoder_chunk_look_back: int = 4
    _decoder_chunk_look_back: int = 1

    _transcription_thread: threading.Thread = None
    _audio_queue: queue.Queue = None
    _stop_event: threading.Event = None
    _audio_cache: np.ndarray = np.array([], dtype=np.float64) 
    _text_cache: str = ""

    _is_busy = False
    _websocket = None
    _stats = OddAsrStats()
    task_id = None

    def __init__(self, 
                 mode="stream", 
                 hotwords="", 
                 audio_rec_filename="",
                 result_callback=None,
                 return_raw_text=True, 
                 is_final=True, 
                 sentence_timestamp=False, 
                 chunk_size=[0, 10, 5], 
                 encoder_chunk_look_back=4, 
                 decoder_chunk_look_back=1,
                 ):
        self._mode = mode  # mode should be a string like 'file','stream', 'pipeline'
        self._hotwords = hotwords  # hotwords should be a string like 'word1 word2'
        self._rec_file = audio_rec_filename  # audio_rec_filename should be a string like 'audio.wav'
        self._result_callback = result_callback

        self._return_raw_text=return_raw_text #return raw text or not, default is True, if False, return json format result, like: {text: "hello world", timestamp: [[0, 1000], [1000, 2000]]}, 
        self._is_final=is_final  #is_final=True, if False, return partial result, like: {text: "hello world", timestamp: [[0, 1000], [1000, 2000]], is_final: False},
        self._sentence_timestamp=sentence_timestamp  #sentence_timestamp=False, if True, return sentence timestamp, like: {text: "hello world", timestamp: [[0, 1000], [1000, 2000]], is_final: False, sentence_timestamp: [[0, 2000]]},

        self._chunk_size = [0, 10, 5] #[0, 10, 5] 600ms, [0, 8, 4] 480ms
        self._chunk_size = chunk_size  #chunk_size[0] is the first chunk size, in ms, 0 means no chunking, -1 means adaptive chunking

        self._encoder_chunk_look_back = encoder_chunk_look_back #number of chunks to lookback for encoder self-attention
        self._decoder_chunk_look_back = decoder_chunk_look_back #number of encoder chunks to lookback for decoder cross-attention

        self._audio_queue = queue.Queue()
        self._stop_event = threading.Event()

        # free resource timeout, auto free resource if N seconds without receiving audio
        self._free_resource_timeout = config.odd_asr_cfg["asr_stream_cfg"]["free_resource_timeout"]
        if self._free_resource_timeout < 0:
            self._free_resource_timeout = 5

        # force to generate final result after N seconds
        self._force_final_result = config.odd_asr_cfg["asr_stream_cfg"]["force_final_result"]
        if self._force_final_result < 0:
            self._force_final_result = 20

        # force trigger punctuate if time elapses less than the configuration value
        self.punct_time_mini_force_trigger = config.odd_asr_cfg["asr_stream_cfg"]["punct_time_mini_force_trigger"]
        if self.punct_time_mini_force_trigger < 0:
            self.punct_time_mini_force_trigger = 3

        # minimum length of punctuation to trigger punctuate
        self.punct_mini_len = config.odd_asr_cfg["asr_stream_cfg"]["punct_mini_len"]
        if self.punct_mini_len < 0:
            self.punct_mini_len = 10

        # check chunk_size is valid, chunk_size[0] is the first chunk size, in ms, 0 means no chunking, -1 means adaptive chunking, chunk_size[1] is the chunk size, in ms, chunk_size[2] is the overlap size, in ms
        if len(self._chunk_size) != 3:
            raise ValueError("chunk_size should be a list of 3 elements, chunk_size[0] is the first chunk size, in ms, 0 means no chunking, -1 means adaptive chunking, chunk_size[1] is the chunk size, in ms, chunk_size[2] is the overlap size, in ms")
        if self._chunk_size[0] < -1 or self._chunk_size[0] > 60000:
            raise ValueError("chunk_size[0] should be between -1 and 60000, in ms")
        if self._chunk_size[1] < 0 or self._chunk_size[1] > 60000:
            raise ValueError("chunk_size[1] should be between 0 and 60000, in ms")
        if self._chunk_size[2] < 0 or self._chunk_size[2] > 60000:
            raise ValueError("chunk_size[2] should be between 0 and 60000, in ms")
        
    def _default_callback(self, result):

        print(result)

class OddAsrStream:
    vad_model = None
    stream_model = None
    punc_model = None
    streamParam: OddAsrParamsStream = None
    device: str = "cuda:0"
    lock: threading.Lock = None

    def __init__(self, streamParam:OddAsrParamsStream=None):
        # 初始化 OddAsrParamsStream 实例
        if streamParam is None:
            self.streamParam = OddAsrParamsStream()
        else:
            self.streamParam = streamParam

        self.vad_model = None
        self.stream_model = None
        self.punc_model = None

        # auto detect GPU device
        if config.odd_asr_cfg["enable_gpu"]:
            self.device = "cuda:0" if torch.cuda.is_available() else "cpu"
        else:
            self.device = "cpu"

        if config.odd_asr_cfg["preload_model"]:
            self._load_stream_model(self.device)

        self.lock = threading.Lock()  # 初始化锁

    def set_busy(self, is_busy):
        with self.lock:  # 使用锁保护共享资源
            self.streamParam._is_busy = is_busy
            if not is_busy:
                logger.info(f"set_busy to False, clear _stop_event, websocket={self.streamParam._websocket}, task_id={self.streamParam.task_id}")
                self.streamParam._stop_event.set()
                self.streamParam._transcription_thread.join()
                self.streamParam._transcription_thread = None
                self.streamParam._audio_queue.empty()
                logger.info(f"set_busy to False, clear _stop_event,done")

    def is_busy(self):
        with self.lock:  # 使用锁保护共享资源
            return self.streamParam._is_busy
        
    def set_websocket(self, websocket):
        with self.lock:  # 使用锁保护共享资源
            self.streamParam._websocket = websocket
    
    def get_websocket(self):
        with self.lock:  # 使用锁保护共享资源
            return self.streamParam._websocket
        
    def set_session_id(self, task_id):
        with self.lock:  # 使用锁保护共享资源
            self.streamParam.task_id = task_id

    def get_session_id(self):
        with self.lock:  # 使用锁保护共享资源
            return self.streamParam.task_id

    def _load_stream_model(self, device="cuda:0"):
        # load stream model
        if not self.stream_model:
            self.stream_model = AutoModel(
                model="paraformer-zh-streaming", model_revision="v2.0.4",

                # vad_model='iic/speech_fsmn_vad_zh-cn-16k-common-pytorch', vad_model_revision="v2.0.4",
                # vad_model="fsmn-vad", vad_model_revision="v2.0.4",

                # punc_model='iic/punc_ct-transformer_cn-en-common-vocab471067-large', punc_model_revision="v2.0.4",
                # punc_model='iic/punc_ct-transformer_zh-cn-common-vad_realtime-vocab272727-large', punc_model_revision="v2.0.4",
                # punc_model='iic/punc_ct-transformer_zh-cn-common-vad_realtime-vocab272727', punc_model_revision="v2.0.4",

                # spk_model="cam++",
                log_level="debug",
                hub="ms",  # hub：表示模型仓库，ms为选择modelscope下载，hf为选择huggingface下载。
                device=device,
                disable_update=True,
            )

        if not self.punc_model:
            '''
            from modelscope.pipelines import pipeline
            from modelscope.utils.constant import Tasks

            inference_pipeline = pipeline(
                task=Tasks.punctuation,
                model='damo/punc_ct-transformer_zh-cn-common-vad_realtime-vocab272727'
            )

            inputs = "跨境河流是养育沿岸|人民的生命之源长期以来为帮助下游地区防灾减灾中方技术人员|在上游地区极为恶劣的自然条件下克服巨大困难甚至冒着生命危险|向印方提供汛期水文资料处理紧急事件中方重视印方在跨境河流问题上的关切|愿意进一步完善双方联合工作机制|凡是|中方能做的我们|都会去做而且会做得更好我请印度朋友们放心中国在上游的|任何开发利用都会经过科学|规划和论证兼顾上下游的利益"
            vads = inputs.split("|")
            rec_result_all="outputs:"
            param_dict = {"cache": []}
            for vad in vads:
                rec_result = inference_pipeline(text_in=vad, param_dict=param_dict)
                rec_result_all += rec_result['text']

            print(rec_result_all)            
            '''

            self.punc_model = AutoModel(
                # model='iic/punc_ct-transformer_zh-cn-common-vad_realtime-vocab272727-large', punc_model_revision="v2.0.4",
                model="iic/punc_ct-transformer_zh-cn-common-vad_realtime-vocab272727", model_revision="v2.0.4",
                hub="ms",  # hub：表示模型仓库，ms为选择modelscope下载，hf为选择huggingface下载。
                device=device,
                disable_update=True,
            )


    def transcribe_stream(self, audio_frame, socket, task_id):
        '''
        transcribe audio stream, support real-time transcription, 
        and return partial result, like: 
            {
                text: "hello world", 
                timestamp: [[0, 1000], [1000, 2000]], 
                is_final: False, 
                sentence_timestamp: [[0, 2000]]
            },
        and return final result, like:
            {
                text: "hello world",
                timestamp: [[0, 1000], [1000, 2000]],
                is_final: True,
                sentence_timestamp: [[0, 2000]]
            }

        1. start a thread to transcribe audio stream
        2. add input audio_frame to queue
        3. empty queue, yield result and stop transcribe thread if got an EOF
        '''
        try:
            # 初始化num_chunks变量，确保在任何情况下都有定义
            num_chunks = 0
            
            if self.streamParam._audio_queue is None:
                logger.error(f"_audio_queue is None")
                return ""
            if self.streamParam._stop_event is None:
                logger.error(f"_stop_event is None")
                return ""
            if self.stream_model is None:
                self._load_stream_model()
            
            # Create and start the transcription thread
            if self.streamParam._transcription_thread is None or not self.streamParam._transcription_thread.is_alive():
                self.streamParam._stop_event.clear()  # 确保 _stop_event 未被设置
                # self.streamParam._transcription_thread = threading.Thread(target=self._transcribe_thread_wrapper, args=(self.streamParam,))
                self.streamParam._transcription_thread = threading.Thread(target=self._transcribe_thread_wrapper)
                self.streamParam._transcription_thread.daemon = True  # 设置为守护线程

                logger.info(f"start transcription_thread, websocket={socket}, task_id={task_id}")

                self.streamParam._transcription_thread.start()
                self.streamParam._stats.start_time = time.time()
            # else:
            #     logger.info(f"transcription_thread is running, websocket={socket}, task_id={task_id}")

            # DONT terminite the thread, just add an empty audio_frame to queue, let the previous frames to be processed
            if audio_frame is None:  # Receive EOF signal
                if self.streamParam._audio_cache.size > 0:
                    # 直接将numpy数组放入队列（无需转换为bytes）
                    cache_array = (self.streamParam._audio_cache * 32768).astype(np.int16)
                    frame = AudioFrame(data=cache_array)
                    self.streamParam._audio_queue.put(frame)
                    self.streamParam._audio_cache = np.array([], dtype=np.float64)  # 清空缓存
                # 放入EOF信号
                frame = AudioFrame(data=None)
                self.streamParam._audio_queue.put(frame)
            else:
                if type(audio_frame) is not bytes:
                    logger.error(f"audio_frame is not bytes, type={type(audio_frame)}")
                    return ""

                # Convert bytes to a NumPy array of int16
                pcm_array = np.frombuffer(audio_frame, dtype=np.int16)
                # Convert the array to float64 and normalize it
                new_audio_array = pcm_array.astype(np.float64) / 32768.0

                # Ensure new_audio_array is 1-dimensional
                if new_audio_array.ndim == 0:
                    new_audio_array = np.array([new_audio_array], dtype=np.float64)
                
                chunk_stride = self.streamParam._chunk_size[1] * 960 # 600ms

                # 合并缓存和新音频数组
                with self.lock:  # 使用锁保护共享资源访问
                    combined_data = np.concatenate([self.streamParam._audio_cache, new_audio_array])
                    data_len = len(combined_data)

                # logger.debug(f"transcribe_stream, chunk_stride={chunk_stride}, pcm_array_len={len(pcm_array)}, new_audio_array_len={len(new_audio_array)}, data_len={data_len}")

                # 按chunk_stride分割numpy数组
                if data_len >= chunk_stride:
                    with self.lock:  # 使用锁保护共享资源访问
                        num_chunks = data_len // chunk_stride
                        # 处理完整块
                        for i in range(num_chunks):
                            start = i * chunk_stride
                            end = start + chunk_stride
                            chunk_data = combined_data[start:end].copy()  # 创建数据副本
                            time_offset = time.time() - self.streamParam._stats.start_time

                            frame = AudioFrame(data=chunk_data, timestamp=time_offset)
                            self.streamParam._audio_queue.put(frame)
                        # 保存剩余数据到缓存中
                        self.streamParam._audio_cache = combined_data[num_chunks * chunk_stride:].copy()  # 创建数据副本
                else:
                    # 数据不足chunk_stride，存入缓存
                    with self.lock:  # 使用锁保护共享资源访问
                        self.streamParam._audio_cache = combined_data.copy()  # 创建数据副本

                self.streamParam._stats.total_audio_recv_len += len(audio_frame)
                self.streamParam._stats.last_recv_timestamp = time.time()

        except Exception as e:
            logger.error(f"Error in transcribe_stream: {e}")
            self.streamParam._stop_event.set()  # Stop the thread in case of an error
            if self.streamParam._transcription_thread is not None and self.streamParam._transcription_thread.is_alive():
                self.streamParam._transcription_thread.join()  # Wait for the thread to finish
                self.streamParam._transcription_thread = None
            raise RuntimeError(f"Error processing audio stream: {e}")

    def _retrieve_words_array(self, pkt_timestamp_start, pkt_timestamp_end, text_result):
        words = []

        if isinstance(text_result, list) and len(text_result) > 0 and isinstance(text_result[0], dict) and 'words' in text_result[0]:
            words_array = text_result[0]['words']
            for word_array in words_array:
                words.append({
                    'word': word_array['word'],
                    'begin_time': word_array['begin_time'],
                    'end_time': word_array['end_time'],
                })
        else:
            words.append({
                'word': text_result[0]['text'],
                'begin_time': pkt_timestamp_start,
                'end_time': pkt_timestamp_end,
            })

        return words

    def _save_audio_rec(self, filename, audio_data, sample_rate=16000):
        try:
            # 保存文件
            if not os.path.exists(os.path.dirname(filename)):
                os.makedirs(os.path.dirname(filename))

            # 确保 audio_data 是 NumPy 数组
            if not isinstance(audio_data, np.ndarray):
                audio_data = np.array(audio_data)

            # 若 audio_data 是浮点数类型，转换回 16 位有符号整数
            if audio_data.dtype == np.float64:
                audio_data = (audio_data * 32767).astype(np.int16)

            # 直接写入二进制文件
            with open(filename, 'ab') as binfile:
                audio_data.tofile(binfile)
        except Exception as e:
            logger.error(f"Error saving audio file: {e}")

    def _generate_vad_data(self, pcm_chunk, sd_sentences = [], sr=16000):
        if not self.vad_model:  # 如果没有VAD模型，则不进行VAD处理
            return None
        vad_result = self.vad_model.vad(pcm_chunk)

        if not vad_result.is_speech:
            logger.debug(f"No speech detected, skipping...")

        logger.info(f"VAD result: {vad_result}")


    def _generate_stream_asr(self, pcm_chunk, is_final = False, cache=None, hotwords=""):
        if not self.stream_model:
            return None, cache
        
        text = None
        try:
            # logger.debug(f"_generate_stream_asr:is_final={is_final}, pcm_chunk={type(pcm_chunk)}, len={len(pcm_chunk)}, shape={pcm_chunk.shape}, dtype={pcm_chunk.dtype}")

            text = self.stream_model.generate(input=pcm_chunk, 
                                                cache=cache, 
                                                is_final=is_final, 
                                                # return_raw_text=True,
                                                sentence_timestamp=True,
                                                # use_punc=True,
                                                # punc_threshold=0.5,
                                                hotword=hotwords, 
                                                chunk_size=self.streamParam._chunk_size, 
                                                encoder_chunk_look_back=self.streamParam._encoder_chunk_look_back, 
                                                decoder_chunk_look_back=self.streamParam._decoder_chunk_look_back
                                                )
            
            logger.debug(f"_generate_stream_asr: text_result={text}")

        except Exception as e:
            logger.error(f"Error transcribing audio chunk: {e}")
            text = None
        
        return text, cache

    def _generate_punctuation_input(self, text):
        if self.punc_model is None:
            return text

        try:
            input_text = text

            '''
            [
                {
                    'key': 'rand_key_CwYyBZFyUoYmC', 
                    'text': '一二三四五六七八七六五四三二一一三四五三三。打老虎传前明月光 疑是地上霜', 
                    'punc_array': tensor([1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3])
                }
            ]
            '''
            punc_result = self.punc_model.generate(input=input_text, punc_threshold=0.5)
            logger.info(f"punctuation: \n\tinput={input_text}\n\tpunc_result={punc_result}")
            punc_result_text = punc_result[0]['text']

            # find the specified punctuations in the punc_result
            punct_pattern = re.compile(r'[。？！]')
            matches = list(punct_pattern.finditer(punc_result_text))

            if matches:
                # locate the last punctuation
                last_match = matches[-1]
                split_pos = last_match.end()
                current_text = punc_result_text[:split_pos]
                remaining_text = punc_result_text[split_pos:]

                return current_text, remaining_text
            else:
                return None, punc_result_text

        except Exception as e:
            logger.error(f"Error in punctuation process: {e}")

    def _release(self):
        # self.set_busy(False)
        # self.set_session_id(None)
        # self.set_websocket(None)
        self.streamParam._stop_event.set()
        self.streamParam._audio_queue.put(None)
        # self.streamParam._audio_queue.join()
        # self.streamParam._audio_queue.task_done()
        self.streamParam._stats.reset()

    def _transcribe_thread_wrapper(self):
        import asyncio
        # Create a new event loop for this thread
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        # chunk_size = [0, 10, 5] #[0, 10, 5] 600ms, [0, 8, 4] 480ms
        # encoder_chunk_look_back = 4 #number of chunks to lookback for encoder self-attention
        # decoder_chunk_look_back = 1 #number of encoder chunks to lookback for decoder cross-attention
        is_final = False
        cache = {}
        hotwords = ""
        websocket = None

        tasks = []  # 用于存储所有异步任务

        if self.streamParam._hotwords is not None and self.streamParam._hotwords != "":
            hotwords = self.streamParam._hotwords.split(" ")

        try:
            while not self.streamParam._stop_event.is_set():
                # STEP 1. read from queue, timeout 1 second
                try:
                    frame: AudioFrame = self.streamParam._audio_queue.get(timeout=1)
                    if frame is None:
                        continue
                    logger.debug(f"AudioFrame timestamp={frame.frame_timestamp}, queue length: {self.streamParam._audio_queue.qsize()},websocket={self.streamParam._websocket}")
                except queue.Empty:  # sleep 100ms if read timeout
                    time.sleep(0.1)
                    timediff = time.time() - self.streamParam._stats.last_recv_timestamp
                    # 5 seconds without receiving audio, stop the thread
                    if timediff > self.streamParam._free_resource_timeout:
                        logger.warn(f"No audio received for {self.streamParam._free_resource_timeout} seconds, stopping transcription thread.")
                        is_final = True
                        break
                    continue

                if frame.data is None:  # EOF received
                    logger.warn(f"Received EOF signal, stopping transcription thread.")
                    is_final = True
                    # break

                # STEP 2. save the pcm to a record file
                if config.odd_asr_cfg["asr_stream_cfg"]["save_audio"]:
                    if self.streamParam._rec_file == "":
                        self.streamParam._rec_file = "tmp/" + self.streamParam.task_id + ".pcm"
                    logger.debug(f"save audio frame to {self.streamParam._rec_file}, sr={frame.sr}, len={len(frame.data)}")
                    self._save_audio_rec(self.streamParam._rec_file, frame.data, frame.sr)

                speech = frame.data.copy()  # 创建数据副本
                websocket = self.streamParam._websocket

                # STEP 3. Spit the audio to chunks, each chunk should match the chunk_size initialized in streamParam 
                chunk_stride = self.streamParam._chunk_size[1] * 960 # 600ms
                total_chunk_num = int(len(speech)/chunk_stride)
                # logger.info(f"Processing frame, stride: {chunk_stride}, data={len(speech)}, total_chunk_num={total_chunk_num}, is_final={is_final}")

                # 在_transcribe_thread_wrapper方法中，添加对音频块的有效性检查
                for i in range(total_chunk_num):
                    start = i * chunk_stride
                    end = start + chunk_stride
                    audio_chunk = speech[start:end].copy()  # 创建数据副本
                    
                    # 添加防御性检查
                    if len(audio_chunk) == 0:
                        logger.warning(f"Empty audio chunk at index {i}, skipping...")
                        continue
                    
                    # 检查audio_chunk是否为有效的numpy数组
                    if not isinstance(audio_chunk, np.ndarray) or audio_chunk.ndim != 1:
                        logger.warning(f"Invalid audio chunk format at index {i}, shape: {audio_chunk.shape if isinstance(audio_chunk, np.ndarray) else type(audio_chunk)}")
                        continue
                
                    ## STEP 3.1 VAD
                    vad_result = self._generate_vad_data(audio_chunk)
                    if vad_result and not vad_result.is_speech:
                        logger.debug(f"No speech detected in chunk {i}, skipping...")
                        continue

                    # STEP 4. Transcribe the audio chunk

                    ## stats: total_audio_input_len
                    self.streamParam._stats.total_audio_input_len += len(audio_chunk)
                    ## stats: pkt_timestamp
                    pkt_timestamp_start = frame.frame_timestamp  + i * chunk_stride / frame.sr
                    pkt_timestamp_end = pkt_timestamp_start + chunk_stride / frame.sr / total_chunk_num * (i+1) - i * chunk_stride / frame.sr

                    text = None
                    words = []
                    text_result, cache = self._generate_stream_asr(audio_chunk, is_final, cache, hotwords)

                    if isinstance(text_result, list) and len(text_result) > 0 and isinstance(text_result[0], dict) and 'text' in text_result[0]:
                        text = text_result[0]['text'].strip()
                        if len(text) == 0:
                            text = None
                    else:
                        text = None

                    if text is None:
                        continue
                    elif text is not None and len(text) > 0:
                        words = self._retrieve_words_array(pkt_timestamp_start, pkt_timestamp_end, text_result)
                        self.streamParam._text_cache += text
                        self.streamParam._stats.last_reply_par_timestamp = time.time()
                        logger.info(f"text in cache: {self.streamParam._text_cache}, start time: {pkt_timestamp_start}, end time: {pkt_timestamp_end}")
                        result = OddAsrStreamResult(websocket, self.streamParam._text_cache, is_final=False, index=self.streamParam._stats.index, begin_time=pkt_timestamp_start, end_time=pkt_timestamp_end, is_last=is_final, words=words)
                        enque_asr_result(result)
                    else:
                        logger.warning(f"Unexpected text format at index {i}: {text}")
                        continue

                    # dont input punc model if current length is less than the configuration value
                    # force trigger punctuate if time elapses less than the configuration value
                    time_diff = time.time() - self.streamParam._stats.start_time
                    if time_diff < self.streamParam.punct_time_mini_force_trigger:
                        # if the length of the text is less than the configuration value, skip
                        if len(self.streamParam._text_cache) < self.streamParam.punct_mini_len:
                            continue

                    # STEP 5. Generate the punctuations
                    punct_text, remain_text = self._generate_punctuation_input(self.streamParam._text_cache)
                    if websocket is not None:
                        if punct_text and len(punct_text) > 1:
                            self.streamParam._stats.last_reply_fin_timestamp = time.time()
                            result1 = OddAsrStreamResult(websocket, punct_text, is_final=True, index=self.streamParam._stats.index, begin_time=pkt_timestamp_start, end_time=pkt_timestamp_end)
                            enque_asr_result(result1)

                        if remain_text and len(remain_text) > 0:
                            self.streamParam._stats.last_reply_par_timestamp = time.time()
                            result2 = OddAsrStreamResult(websocket, remain_text, is_final=False, index=self.streamParam._stats.index, begin_time=pkt_timestamp_start, end_time=pkt_timestamp_end)
                            enque_asr_result(result2)

                            # remove punctuations from the remaining text
                            self.streamParam._text_cache = re.sub(r'[。？！，；：“”‘’（）【】{}、,;:"\'()\[\]{}]', '', remain_text)

                            logger.info(f"current_text={remain_text}, after remove punctuations={self.streamParam._text_cache}")

                    else:
                        self.streamParam._text_cache = remain_text

                time.sleep(0.1)

            logger.info(f"Transcription thread stopped.")

            if websocket is not None:
                self.streamParam._stats.last_reply_par_timestamp = time.time()
                result = OddAsrStreamResult(websocket, "END", is_final=True, is_last=True, begin_time=pkt_timestamp_start, end_time=pkt_timestamp_end)
                enque_asr_result(result)

            if tasks:
                loop.run_until_complete(asyncio.gather(*tasks))

        except Exception as e:
            logger.error(f"Error in transcription thread: {e}")
        finally:
            logger.info(f"Transcription thread stopped.")
            loop.close()
            self._release()


# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: odd_asr_app.py 
@info: 消息模版
"""

import werkzeug.utils
import os
from datetime import timedelta
from flask import Flask, request, jsonify, make_response

from oddasr.logic import odd_asr_exceptions 
import oddasr.odd_asr_config as config

# from oddasr.log import logger

# register blueprints
def register_blueprints(new_app, path):
    for name in werkzeug.utils.find_modules(path):
        m = werkzeug.utils.import_string(name)
        new_app.register_blueprint(m.bp)
    new_app.errorhandler(odd_asr_exceptions.CodeException)(odd_asr_exceptions.handler)
    return new_app

app = Flask(__name__, static_url_path='')
register_blueprints(app, 'oddasr.router')
app.config['SECRET_KEY'] = os.urandom(24)
app.config['PERMANENT_SESSION_LIFETIME'] = timedelta(days=7)

# 添加全局缓存控制中间件
@app.after_request
def add_cache_control(response):
    # 禁止浏览器缓存所有响应
    response.headers['Cache-Control'] = 'no-store, no-cache, must-revalidate, post-check=0, pre-check=0, max-age=0'
    response.headers['Pragma'] = 'no-cache'
    response.headers['Expires'] = '-1'
    return response

import oddasr.router.asr_api



import os
from oddasr.logic.odd_asr_file import OddAsrFile
from pynput import keyboard

asr = OddAsrFile()

def on_press(key):
    try:
        # 检测空格键
        if key == keyboard.Key.space:
            print("Space pressed. Starting recognition...")
            result = asr.transcribe_file("./test_cn_male_9s.wav")
            print(f"Recognition Result:\n{result}")
        # 检测退出键 'q'
        elif key.char == 'q':
            print("Quitting program.")
            return False
        else:
            print(f"Key pressed: {key.char}, but not recognized.")
    except Exception as e:
        print(f"Error: {e}")

def main():
    print("Press SPACE to recognize './test_cn_male_9s.wav', or 'q' to quit.")

    # 使用 pynput 监听按键
    with keyboard.Listener(on_press=on_press) as listener:
        listener.join()

if __name__ == "__main__":
    main()


import json
import os

# 配置文件路径
CONFIG_FILE = os.path.join(os.path.dirname(__file__), 'config.json')

# 默认配置
DEFAULT_CONFIG = {
    "HOST": "0.0.0.0",
    "PORT": 9002,
    "WS_HOST": "0.0.0.0",
    "WS_PORT": 8101,
    "Debug": False,
    "odd_asr_cfg": {
        "preload_model": True,
        "enable_gpu": False,
        "disable_stream": False,
        "concurrent_thread": 0,
        "asr_stream_cfg": {
            'max_instance': 1,
            'save_audio': False,
            'punct_mini_len': 10,
            'punct_time_mini_force_trigger': 3,
            'free_resource_timeout': 5,
            'force_final_result': 20,
            'vad_threshold': 0.8,
            'vad_min_speech_duration': 300,
            'vad_min_silence_duration': 200
        },
        "asr_file_cfg": { 'max_instance':1, 'save_audio': False },
        "asr_sentence_cfg": {
            'max_instance': 1,
            'save_audio': False,
            'punct_mini_len': 10,
            'punct_time_mini_force_trigger': 3,
            'free_resource_timeout': 5,
            'force_final_result': 20,
            'vad_threshold': 0.8,
            'vad_min_speech_duration': 300,
            'vad_min_silence_duration': 200
        },
        "enable_https": False,
        "ssl_cert_path": "scripts/cert.pem",
        "ssl_key_path": "scripts/key.pem",
    },
    "odd_asr_slp_cfg": {
        "sensitive_words": {
            "path": "scripts/sensitivewords",
        },
        "hotwords": {
            "path": "scripts/hotwords",
        }
    },
    "db_cfg": {
        "db_engine": "sqlite",
        "db_name": "oddasr.db",
        "db_user": "",
        "db_password": "",
        "db_host": "",
        "db_port": "",
    },
    "redis_cfg": {
        "redis_enabled": False,
        "redis_host": "127.0.0.1",
        "redis_port": 7379,
        "redis_password": "",
    },
    "log_file": "oddasr.log",
    "log_path": "logs/",
    "log_level": 10,
    "asr": {'liveasr':'', 'appid':'oddasrtest', 'secret':'oddasrtest'},
    "Users": 'oddasr_users.json'
}

# 加载配置
def load_config():
    try:
        with open(CONFIG_FILE, 'r', encoding='utf-8') as f:
            config = json.load(f)
        return config
    except FileNotFoundError:
        # 如果配置文件不存在，使用默认配置并创建配置文件
        with open(CONFIG_FILE, 'w', encoding='utf-8') as f:
            json.dump(DEFAULT_CONFIG, f, indent=4, ensure_ascii=False)
        return DEFAULT_CONFIG
    except Exception as e:
        print(f"Error loading config file: {e}")
        return DEFAULT_CONFIG

# 加载配置
config_data = load_config()

# 将配置项导出为模块级变量，保持原有接口不变
HOST = config_data.get("HOST", DEFAULT_CONFIG["HOST"])
PORT = config_data.get("PORT", DEFAULT_CONFIG["PORT"])
WS_HOST = config_data.get("WS_HOST", DEFAULT_CONFIG["WS_HOST"])
WS_PORT = config_data.get("WS_PORT", DEFAULT_CONFIG["WS_PORT"])
Debug = config_data.get("Debug", DEFAULT_CONFIG["Debug"])

odd_asr_cfg = config_data.get("odd_asr_cfg", DEFAULT_CONFIG["odd_asr_cfg"])
odd_asr_slp_cfg = config_data.get("odd_asr_slp_cfg", DEFAULT_CONFIG["odd_asr_slp_cfg"])
db_cfg = config_data.get("db_cfg", DEFAULT_CONFIG["db_cfg"])
redis_cfg = config_data.get("redis_cfg", DEFAULT_CONFIG["redis_cfg"])

log_file = config_data.get("log_file", DEFAULT_CONFIG["log_file"])
log_path = config_data.get("log_path", DEFAULT_CONFIG["log_path"])
log_level = config_data.get("log_level", DEFAULT_CONFIG["log_level"])

asr = config_data.get("asr", DEFAULT_CONFIG["asr"])
Users = config_data.get("Users", DEFAULT_CONFIG["Users"])


import sys

import pymysql

import config

'''
用于更新数据库字段.
'''
if __name__ == '__main__':
    # create database
    usr = sys.argv[1]
    pwd = sys.argv[2]
    if not usr or not pwd:
        usr = config.Mysql_User
        pwd = config.Mysql_Pwd

    con = pymysql.connect(host=config.Mysql_Host,
                          user=usr,
                          password=pwd,
                          charset='utf8mb4')
    cur = con.cursor()
    with open('update_table.sql', 'r', True, 'UTF-8') as f:
        sql_list = f.read().split('$$')[:-1]

        for x in sql_list:
            line = x.split('\n')
            L = ''
            for y in line:
                l = y.split('#', 1)[0].split('-- ', 1)[0]
                L = L + ' ' + l
            # sql语句添加分号结尾
            sql_item = L + ';'
            print(sql_item)
            cur.execute(sql_item)
    con.commit()
    cur.close()
    con.close()



# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: result.py 
@time: 2025/6/9 15:00
@info: 消息模版
"""

import queue
import asyncio
from funasr import AutoModel
import threading

from oddasr.log import logger
import oddasr.logic.proto as proto

# 创建一个线程安全的优先级队列
asr_result_queue = queue.SimpleQueue()
asr_result_queue_lock = threading.Lock()

class Result:
    def __init__(self):
        self._result = {}

    def set_code(self, error_code):
        self._result['error_code'] = error_code

    def set_msg(self, error_desc):
        self._result['error_desc'] = error_desc

    def set_data(self, data):
        self._result['data'] = data

    @property
    def result(self):
        return self._result


def from_exc(exc):
    r = Result()
    r.set_code(exc.error_code)
    r.set_msg(exc.error_desc)
    return r.result


def from_data(data):
    r = Result()
    r.set_data(data)
    return r.result


class OddAsrStreamResult:

    def __init__(self, webocket, text, index=0, begin_time=0, end_time=0, is_final=False, is_last=False, words=[]):
        self.webocket = webocket
        self.res = proto.TOddAsrTranscribeRes()
        
        if is_final:
            self.res.header.name = "SentenceEnd"
            # self.cached_text = ""
        else:
            self.res.header.name = "TranscriptionResultChanged"
            # self.cached_text += text

        self.res.payload.result = text
        self.res.payload.begin_time = begin_time
        self.res.payload.end_time = end_time
        self.res.payload.index = index
        self.res.payload.fin = 1 if is_last else 0
        self.res.payload.words = words

def enque_asr_result(message: OddAsrStreamResult):
    global asr_result_queue
    asr_result_queue.put(message)
    
    logger.info(f"asr result queue size={asr_result_queue.qsize()}")

async def notify_task(_wss_server=None):
    global asr_result_queue
    logger.debug(f"=============================================notifyTask: start===================================")
    while True:
        try:
            if not asr_result_queue.empty():
                # print(f"queue size={asr_result_queue.qsize()}")
                message:OddAsrStreamResult = asr_result_queue.get()
                # print(f"notifyTask: {message.text}")

                if _wss_server:
                    if message.webocket in _wss_server._clients_set:
                        # 发送消息给客户端
                        # print(f"notifyTask: send to client={message.webocket}")
                        str = proto.obj_to_dict_recursive(message.res)
                        logger.debug(f"notifyTask: send to client={message.webocket}, res={str}")
                        await _wss_server.doSend(message.webocket, str)
            else:
                # print(f"notifyTask: queue empty")
                # 队列为空，等待一段时间后再检查
                await asyncio.sleep(0.1)
        except Exception as e:
            # 处理其他异常
            logger.error(f"notifyTask error: {e}")
            continue

class notifyTask():
    @staticmethod
    def start(wss_server=None, loop=None):
        # 获取当前事件循环
        if not loop:
            loop = asyncio.get_running_loop()
        # 创建后台线程
        loop.create_task(notify_task(wss_server))
        # 将后台线程设置为守护线程，以便在主线程结束时自动退出
        logger.info("=> Start ASR result dispatch Task Success")





import functools
from flask import session,Blueprint
from werkzeug.utils import redirect

bp = Blueprint('jdl_session', __name__, url_prefix='')

def session_required(view_func):
    @functools.wraps(view_func)
    def verify_session(*args,**kwargs):
        
        if session.get("user") == None:
           return redirect('login.html',code=302);
        
        return view_func(*args,**kwargs)

    return verify_session


# -*- coding: utf-8 -*-
""" 
@author: catherine wei
@contact: EMAIL@contact: catherine@oddmeta.com
@software: PyCharm 
@file: odd_wss_server.py 
@info: 消息模版
"""

"""Server example using the asyncio API."""

import asyncio
from websockets.asyncio.server import serve
import json
import numpy as np
import websockets
import ssl
import uuid
import queue

import oddasr.odd_asr_config as config
from oddasr.log import logger

from oddasr.logic.odd_asr_stream import OddAsrStream, OddAsrParamsStream
from oddasr.logic.odd_asr_result import notifyTask
from oddasr.logic.odd_asr_exceptions import *
from oddasr.logic.proto import TOddAsrTranscribeRes, obj_to_dict, TOddAsrApplyRes, obj_from_dict_recursive, obj_to_dict_recursive

'''
client --> server: TCmdApppyAsrReq
server --> client: TCmdApppyAsrRsp with task_id
server --> server: add client to clients
client --> server: PCMData
server --> client: ASRResult

'''


odd_asr_stream_set = set()
_wss_server = None

def find_free_odd_asr_stream(websocket, task_id):
    '''
    找到一个空闲的odd_asr_stream
    :param websocket:
    :return:
    '''
    for odd_asr_stream in odd_asr_stream_set:
        if not odd_asr_stream.is_busy():
            odd_asr_stream.set_busy(True)
            odd_asr_stream.set_session_id(task_id)
            odd_asr_stream.set_websocket(websocket)

            return odd_asr_stream
        
    return None

def find_odd_asr_stream_by_websocket(websocket):
    '''
    找到websocket对应的odd_asr_stream
    :param websocket:
    :return:
    '''
    for odd_asr_stream in odd_asr_stream_set:
        if odd_asr_stream.get_websocket() == websocket:
            return odd_asr_stream
        
    return None

def free_odd_asr_stream(odd_asr_stream):
    '''
    释放一个odd_asr_stream
    :param odd_asr_stream:
    :return:
    '''
    odd_asr_stream.set_busy(False)
    odd_asr_stream.set_session_id(None)
    odd_asr_stream.set_websocket(None)

def find_odd_asr_stream_by_session_id(task_id):
    '''
    找到一个已存在的odd_asr_stream
    :param task_id:
    :return:
    '''
    for odd_asr_stream in odd_asr_stream_set:
        if odd_asr_stream.get_session_id() == task_id:
            return odd_asr_stream
    return None

class OddWssServer:
    def __init__(self):
        self._clients_set = set()
        self._sessionid_set = set()
        self._conn_sessionid = dict()
        self._sessionid_conn = dict()

    async def handle_client(self, *args, **kwargs):
        # 从参数中提取 websocket 和 path
        websocket = args[0]
        logger.debug(f"Client connected: {websocket}, args={args}, len={len(args)}, kwargs={kwargs}")
        
        try:
            async for message in websocket:
                if not websocket in self._clients_set:
                    '''
                    新连接上来的client，第一个消息必须是一个json.
                    namespace: SpeechTranscriber
                    name: StartTranscription
                    message_id: 消息id
                    token: 鉴权token
                    task_id: 正常第一次连接为空，Apply成功后，由服务端生成并返回给客户端。客户端断线重连的时候带上。
                    '''
                    result, res, task_id = self.doInit(websocket, message)

                    logger.info(f"doInit={result}, res={obj_to_dict_recursive(res)}, websocket={websocket}, task_id={task_id}")

                    await websocket.send(json.dumps(obj_to_dict_recursive(res)))

                    if result:
                        logger.debug(f"add client={websocket} to clients_set")
                    else:
                        logger.error(f"doInit failed, close client={websocket}")
                        await websocket.close()
                        return False

                    # find a odd_asr_stream
                    odd_asr_stream:OddAsrStream = find_odd_asr_stream_by_session_id(task_id=task_id)
                    if odd_asr_stream is None:
                        odd_asr_stream = find_free_odd_asr_stream(websocket, task_id)
                        if odd_asr_stream is None:
                            logger.error(f"no free odd_asr_stream, close client={websocket}")
                            await websocket.close()
                            return
                        else:
                            logger.debug(f"found free odd_asr_stream, task_id={task_id}")
                            self._sessionid_set.add(task_id)
                    else:
                        logger.info(f"found existing odd_asr_stream, task_id={task_id}, websocket={odd_asr_stream.get_websocket()}:{websocket}")

                    self._clients_set.add(websocket)
                    self._conn_sessionid[websocket] = task_id
                    self._sessionid_conn[task_id] = websocket
                else:
                    '''
                    客户端已经申请过ASR，并且已经连接上了，此时收到的消息是PCMData
                    '''
                    if type(message) is bytes:
                        self.onRecv(websocket, message)
                    else:
                        # {'service_type':'ASR','msg_type':'STOP_SESSION_REQ'}
                        self.doCtrl(websocket, message)
                        logger.error(f"invalid message type, {type(message)}, {message}")

                    continue

        finally:
            self.onClose(websocket)

    async def doSend(self, websocket, message):
        '''
        发送消息给客户端
        :param websocket:
        :param message:
        :return:
        '''
        try:
            if not isinstance(message, str):
                message = json.dumps(message)
                logger.debug(f"doSend: {message}")
            else:
                logger.debug(f"doSend: {len(message)}")
            await websocket.send(message)
        except websockets.exceptions.ConnectionClosedOK:
            logger.error(f"Connection closed normally: {websocket}")
        except websockets.exceptions.ConnectionClosedError as e:
            logger.error(f"Connection closed with error: {e}, {websocket}")
        except Exception as e:
            logger.error(f"Unexpected error in doSend: {e}, {websocket}")

    async def doBroadcast(self, message):
        for client in self._clients_set:
            await client.send(message)

    def onRecv(self, websocket, pcm_data):
        # logger.debug(f"onRecv: {len(pcm_data)}, websocket={websocket}")

        # 找到对应的odd_asr_stream
        task_id = ""
        odd_asr_stream: OddAsrStream = find_odd_asr_stream_by_websocket(websocket=websocket)
        if odd_asr_stream:
            task_id = odd_asr_stream.get_session_id()
            # logger.debug(f"find_odd_asr_stream_by_websocket, task_id={task_id}")
            odd_asr_stream.transcribe_stream(pcm_data, socket=websocket, task_id=task_id)
        else:
            logger.error(f"find_odd_asr_stream_by_websocket, not found, websocket={websocket}")

    def onClose(self, websocket):
        logger.warn(f"Client disconnected: {websocket}")
        if websocket in self._clients_set:
            '''
            客户端断开连接，需要从clients中删除。
            然而暂不在sessionid_set中删除，因为可能是客户端断线重连，此时sessionid还存在。
            但是，后面需要做一个计时器，定期检查，若超时未收到客户端的消息，则删除sessionid。
            '''
            task_id = self._conn_sessionid[websocket]

            # self._sessionid_set.remove(task_id)
            self._sessionid_conn.pop(task_id)
            self._conn_sessionid.pop(websocket)
            self._clients_set.remove(websocket)
            odd_asr_stream = find_odd_asr_stream_by_session_id(task_id=task_id)

            # logger.warn(f"remove task_id={task_id}, client={websocket}, odd_asr_stream={odd_asr_stream}")

            if odd_asr_stream:
                odd_asr_stream.set_websocket(None)
                odd_asr_stream.set_busy(False)
                # odd_asr_stream.set_session_id('')

            logger.warn(f"remove task_id={task_id}, client={websocket}")
        else:
            logger.error(f"client={websocket} not in clients_set")

    def doInit(self, websocket, message):
        # 解析json，若第一个消息不是json，则关闭连接
        result = False
        try:
            req = json.loads(message)
        except Exception as e:
            logger.error(f"Invalid json format. Received message: {message}. webocket={websocket}")
            message_id = ''
            name = ''

            res = TOddAsrApplyRes()
            res.header.message_id = message_id
            res.header.name = name
            res.header.status = EM_ERR_ASR_ARGS_ERROR
            res.header.status_text = mai_err_name(EM_ERR_ASR_ARGS_ERROR)

            return result, res, ""

        # 解析json中的session_id
        res = TOddAsrApplyRes()
        res.header.message_id = req['message_id'] if 'message_id' in req else ''
        res.header.task_id = req['task_id'] if 'task_id' in req else ''
        res.header.name = req['name'] if 'name' in req else ''

        # 若第一个消息不是StartTranscription，则关闭连接
        if req['name'] != 'StartTranscription':
            logger.error(f"Invalid name. Received message: {message}, req['name']")
            res.header.status = EM_ERR_ASR_ARGS_ERROR
            res.header.status_text = mai_err_name(EM_ERR_ASR_ARGS_ERROR)

            return result, res, res.header.task_id

        if "task_id" in req and req['task_id']:
            if req['task_id'] in self._sessionid_set:
                '''若task_id已经存在，说明是之前已经申请过ASR，但是中间网络异常，并断线重连了
                '''
                self._sessionid_conn[req['task_id']] = websocket
                self._conn_sessionid[websocket] = req['task_id']

                res.header.status = 0
                res.header.status_text = "Success"

                result = True
            else:
                '''
                若task_id不存在，说明是一个非法请求
                '''
                logger.error(f"Received message: {message}, req['task_id']={req['task_id']}")
                res.header.status  = EM_ERR_ASR_SESSION_ID_NOVALID
                res.header.status_text = mai_err_name(EM_ERR_ASR_SESSION_ID_NOVALID)
                result = False
        else:
            '''
            若task_id为空，说明是第一次申请ASR，需要生成一个task_id
            '''
            res.header.task_id = str(uuid.uuid1())
            res.header.name = "SentenceBegin"
            res.header.status = 0

            self._sessionid_set.add(res.header.task_id)
            self._sessionid_conn[res.header.task_id] = websocket
            self._conn_sessionid[websocket] = res.header.task_id

            result = True

        return result, res, res.header.task_id

    def doCtrl(self, websocket, message):
        # 解析json，若第一个消息不是json，则关闭连接
        result = False
        try:
            req = json.loads(message)
        except Exception as e:
            logger.error(f"Invalid json format. Received message: {message}. webocket={websocket}")
            message_id = ''
            name = ''

            res = TOddAsrApplyRes()
            res.header.message_id = message_id
            res.header.name = name
            res.header.status = EM_ERR_ASR_ARGS_ERROR
            res.header.status_text = mai_err_name(EM_ERR_ASR_ARGS_ERROR)

            return result, res, ""

        # 解析json中的session_id
        name_value = req.get('name', '')
        task_id = req.get('task_id', '')
        message_id = req.get('message_id', '')

        logger.warn(f"Received message: {message}, name={name_value}, task_id={task_id}, message_id={message_id}")

        res = TOddAsrApplyRes()
        res.header.message_id = message_id
        res.header.task_id = task_id
        res.header.name = name_value

        # 若消息不是STOP_SESSION_REQ
        if name_value != 'STOP_SESSION_REQ':
            logger.error(f"Invalid name. Received message: {message}, req['name']={name_value}")
            res.header.status = EM_ERR_ASR_ARGS_ERROR
            res.header.status_text = mai_err_name(EM_ERR_ASR_ARGS_ERROR)

            return result, res, res.header.task_id
        
        odd_asr_stream = find_odd_asr_stream_by_session_id(task_id=task_id)
        if not odd_asr_stream:
            logger.error(f"task_id={task_id} not found")
            res.header.status = EM_ERR_ASR_SESSION_ID_NOVALID
            res.header.status_text = mai_err_name(EM_ERR_ASR_SESSION_ID_NOVALID)
            return result, res, res.header.task_id
        else:
            if task_id in self._sessionid_set:
                self._sessionid_conn[task_id] = None

            self._conn_sessionid[websocket] = None
            res.header.status = 0
            res.header.status_text = "Success"


        result = True

        return result, res, res.header.task_id


    async def send(self, websocket, message):
        await websocket.send(message)

async def notify_task(_wss_server=None):
    global asr_result_queue
    while True:
        try:
            # 使用 asyncio.to_thread 从队列获取消息
            message = await asyncio.to_thread(asr_result_queue.get, timeout=1)
            logger.debug(f"notifyTask: {message.text}")

            if _wss_server:
                if message.webocket in _wss_server._clients_set:
                    # 发送消息给客户端
                    res = TOddAsrTranscribeRes()
                    res.header.message_id = message.message_id
                    res.header.name = message.name
                    res.header.status = message.status
                    res.header.status_text = message.status_text
                    res.payload = message.payload
                    message.text = json.dumps(obj_to_dict_recursive(res))
                    
                    logger.debug(f"notifyTask: send to client={message.webocket}, message.text={message.text}")
                    await _wss_server.doSend(message.webocket, message.text)
        except queue.Empty:
            # 队列为空，继续等待
            continue
        except Exception as e:
            # 处理其他异常
            logger.error(f"notifyTask error: {e}")
            continue


def init_instances_stream(server: OddWssServer = None):
    '''
    初始化odd_asr_stream实例。
    由于初始化加载模型比较耗时，所以在启动的时候就预加载。
    电脑内存太小，默认这里只初始化2个实例。
    每个odd_asr_stream实例对应一个websocket连接。
    每个websocket连接对应一个odd_asr_stream实例。
    每个odd_asr_stream实例对应一个session_id。
    每个session_id对应一个websocket连接。
    每个websocket连接对应一个session_id。
    每个session_id对应一个odd_asr_stream实例。
    :param server:
    :return:
    '''
    max_instance = config.odd_asr_cfg["asr_stream_cfg"]["max_instance"]

    if max_instance <= 0:
        max_instance = 1

    for i in range(max_instance):
        odd_asr_stream_param: OddAsrParamsStream = OddAsrParamsStream(
            mode="stream",
            hotwords="", 
            audio_rec_filename="",
            # result_callback=server.doSend,
        )
        odd_asr_stream = OddAsrStream(odd_asr_stream_param)
        odd_asr_stream_set.add(odd_asr_stream)

def init_notify_task(server: OddWssServer):
    '''
    初始化notifyTask
    :param server:
    :return:
    '''
    notify_Task = notifyTask()
    notify_Task.start(server)


# 修改 oddasr/logic/odd_wss_server.py 文件中的 start_wss_server 函数
async def start_wss_server():
    global _wss_server
    _wss_server = OddWssServer()
    
    ssl_context = None
    if config.odd_asr_cfg["enable_https"]:
        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
        ssl_context.load_cert_chain(
            config.odd_asr_cfg["ssl_cert_path"],
            config.odd_asr_cfg["ssl_key_path"]
        )
    
    # 创建服务器
    server = await serve(_wss_server.handle_client, config.WS_HOST, config.WS_PORT, ssl=ssl_context)
    
    try:
        # 等待取消
        await asyncio.Future()
    except asyncio.CancelledError:
        # 关闭服务器
        server.close()
        await server.wait_closed()
        logger.info("WebSocket server closed")
        raise

    init_notify_task(_wss_server)
    init_instances_stream(_wss_server)

    # Configure SSL context if HTTPS is enabled
    ssl_context = None
    if config.odd_asr_cfg["enable_https"]:
        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
        ssl_context.load_cert_chain(
            config.odd_asr_cfg["ssl_cert_path"],
            config.odd_asr_cfg["ssl_key_path"]
        )
    
    async with serve(_wss_server.handle_client, config.WS_HOST, config.WS_PORT, ssl=ssl_context):
        await asyncio.Future()  # Run forever

if __name__ == "__main__":
    asyncio.run(start_wss_server())